@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_dis_h
-#define ___VBox_dis_h
+#ifndef VBOX_INCLUDED_dis_h
+#define VBOX_INCLUDED_dis_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <VBox/types.h>
 #include <VBox/disopcode.h>
@@ -844,5 +847,5 @@ DISDECL(bool)   DISFormatYasmIsOddEncoding(PDISSTATE pDis);
 
 RT_C_DECLS_END
 
-#endif
+#endif /* !VBOX_INCLUDED_dis_h */
 
