@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,8 +23,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___VBox_vmm_pdmstorageifs_h
-#define ___VBox_vmm_pdmstorageifs_h
+#ifndef VBOX_INCLUDED_vmm_pdmstorageifs_h
+#define VBOX_INCLUDED_vmm_pdmstorageifs_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/sg.h>
 #include <VBox/types.h>
@@ -228,7 +231,7 @@ typedef struct PDMIMEDIAPORT
 
 
     /**
-     * Queries the vendor and product ID and revision to report for INQUIRY commands in underlying devices.
+     * Queries the vendor and product ID and revision to report for INQUIRY commands in underlying devices, optional.
      *
      * @returns VBox status code.
      * @param   pInterface      Pointer to this interface.
@@ -538,6 +541,8 @@ typedef struct PDMIMEDIA
 typedef struct PDMMEDIAEXIOREQINT *PDMMEDIAEXIOREQ;
 /** Pointer to an I/O request handle. */
 typedef PDMMEDIAEXIOREQ *PPDMMEDIAEXIOREQ;
+/** NIL I/O request handle. */
+#define NIL_PDMMEDIAEXIOREQ     ((PDMMEDIAEXIOREQ)0)
 
 /** A I/O request ID. */
 typedef uint64_t PDMMEDIAEXIOREQID;
@@ -946,17 +951,20 @@ typedef struct PDMIMEDIAEX
      * @param   pbCdb           The SCSI CDB containing the command.
      * @param   cbCdb           Size of the CDB in bytes.
      * @param   enmTxDir        Direction of transfer.
+     * @param   penmTxDirRet    Where to store the transfer direction as parsed from the CDB, optional.
      * @param   cbBuf           Size of the transfer buffer.
      * @param   pabSense        Where to store the optional sense key.
      * @param   cbSense         Size of the sense key buffer.
+     * @param   pcbSenseRet     Where to store the amount of sense data written, optional.
      * @param   pu8ScsiSts      Where to store the SCSI status on success.
      * @param   cTimeoutMillies Command timeout in milliseconds.
      * @thread  Any thread.
      */
-    DECLR3CALLBACKMEMBER(int, pfnIoReqSendScsiCmd,(PPDMIMEDIAEX pInterface, PDMMEDIAEXIOREQ hIoReq, uint32_t uLun,
-                                                   const uint8_t *pbCdb, size_t cbCdb, PDMMEDIAEXIOREQSCSITXDIR enmTxDir,
-                                                   size_t cbBuf, uint8_t *pabSense, size_t cbSense, uint8_t *pu8ScsiSts,
-                                                   uint32_t cTimeoutMillies));
+    DECLR3CALLBACKMEMBER(int, pfnIoReqSendScsiCmd,(PPDMIMEDIAEX pInterface, PDMMEDIAEXIOREQ hIoReq,
+                                                   uint32_t uLun, const uint8_t *pbCdb, size_t cbCdb,
+                                                   PDMMEDIAEXIOREQSCSITXDIR enmTxDir, PDMMEDIAEXIOREQSCSITXDIR *penmTxDirRet,
+                                                   size_t cbBuf, uint8_t *pabSense, size_t cbSense, size_t *pcbSenseRet,
+                                                   uint8_t *pu8ScsiSts, uint32_t cTimeoutMillies));
 
     /**
      * Returns the number of active I/O requests.
@@ -1032,10 +1040,10 @@ typedef struct PDMIMEDIAEX
 
 } PDMIMEDIAEX;
 /** PDMIMEDIAEX interface ID. */
-#define PDMIMEDIAEX_IID                      "1f82b709-a9f7-4928-ad50-e879c9bbeba1"
+#define PDMIMEDIAEX_IID                      "29c9e82b-934e-45c5-bb84-0d871c3cc9dd"
 
 /** @} */
 
 RT_C_DECLS_END
 
-#endif
+#endif /* !VBOX_INCLUDED_vmm_pdmstorageifs_h */
