@@ -4,24 +4,28 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
  *
- * The contents of this file may alternatively be used under the terms
- * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
- * CDDL are applicable instead of those of the GPL.
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
  *
- * You may elect to license modified versions of this file under the
- * terms and conditions of either the GPL or the CDDL or both.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
  */
 
 
@@ -291,62 +295,59 @@ static void vbglPhysHeapExcludeBlock (VBGLPHYSHEAPBLOCK *pBlock)
     pBlock->pPrev = NULL;
 }
 
-static VBGLPHYSHEAPBLOCK *vbglPhysHeapChunkAlloc (uint32_t cbSize)
+static VBGLPHYSHEAPBLOCK *vbglPhysHeapChunkAlloc(uint32_t cbMinBlock)
 {
-    RTCCPHYS physAddr;
+    RTCCPHYS           PhysAddr = NIL_RTHCPHYS;
     VBGLPHYSHEAPCHUNK *pChunk;
-    VBGLPHYSHEAPBLOCK *pBlock;
-    VBGL_PH_dprintf(("Allocating new chunk of size %d\n", cbSize));
+    uint32_t           cbChunk;
+    VBGL_PH_dprintf(("Allocating new chunk for %#x byte allocation\n", cbMinBlock));
+    AssertReturn(cbMinBlock < _128M, NULL); /* paranoia */
 
-    /* Compute chunk size to allocate */
-    if (cbSize < VBGL_PH_CHUNKSIZE)
+    /* Compute the size of the new chunk, rounding up to next chunk size,
+       which must be power of 2. */
+    Assert(RT_IS_POWER_OF_TWO(VBGL_PH_CHUNKSIZE));
+    cbChunk = cbMinBlock + sizeof(VBGLPHYSHEAPCHUNK) + sizeof(VBGLPHYSHEAPBLOCK);
+    cbChunk = RT_ALIGN_32(cbChunk, VBGL_PH_CHUNKSIZE);
+
+    /* This function allocates physical contiguous memory below 4 GB.  This 4GB
+       limitation stems from using a 32-bit OUT instruction to pass a block
+       physical address to the host. */
+    pChunk = (VBGLPHYSHEAPCHUNK *)RTMemContAlloc(&PhysAddr, cbChunk);
+    if (pChunk)
     {
-        /* Includes case of block size 0 during initialization */
-        cbSize = VBGL_PH_CHUNKSIZE;
+        VBGLPHYSHEAPCHUNK *pOldHeadChunk;
+        VBGLPHYSHEAPBLOCK *pBlock;
+        AssertRelease(PhysAddr < _4G && PhysAddr + cbChunk <= _4G);
+
+        /* Init the new chunk. */
+        pChunk->u32Signature     = VBGL_PH_CHUNKSIGNATURE;
+        pChunk->cbSize           = cbChunk;
+        pChunk->physAddr         = (uint32_t)PhysAddr;
+        pChunk->cAllocatedBlocks = 0;
+        pChunk->pNext            = NULL;
+        pChunk->pPrev            = NULL;
+
+        /* Initialize the free block, which now occupies entire chunk. */
+        pBlock = (VBGLPHYSHEAPBLOCK *)(pChunk + 1);
+        vbglPhysHeapInitBlock(pBlock, pChunk, cbChunk - sizeof(VBGLPHYSHEAPCHUNK) - sizeof(VBGLPHYSHEAPBLOCK));
+        vbglPhysHeapInsertBlock(NULL, pBlock);
+
+        /* Add the chunk to the list. */
+        pOldHeadChunk = g_vbgldata.pChunkHead;
+        pChunk->pNext = pOldHeadChunk;
+        if (pOldHeadChunk)
+            pOldHeadChunk->pPrev = pChunk;
+        g_vbgldata.pChunkHead    = pChunk;
+
+        VBGL_PH_dprintf(("Allocated chunk %p LB %#x, block %p LB %#x\n", pChunk, cbChunk, pBlock, pBlock->cbDataSize));
+        return pBlock;
     }
-    else
-    {
-        /* Round up to next chunk size, which must be power of 2 */
-        cbSize = (cbSize + (VBGL_PH_CHUNKSIZE - 1)) & ~(VBGL_PH_CHUNKSIZE - 1);
-    }
-
-    physAddr = 0;
-    /* This function allocates physical contiguous memory (below 4GB) according to the IPRT docs.
-     * Address < 4G is required for the port IO.
-     */
-    pChunk = (VBGLPHYSHEAPCHUNK *)RTMemContAlloc (&physAddr, cbSize);
-
-    if (!pChunk)
-    {
-        LogRel(("vbglPhysHeapChunkAlloc: failed to alloc %u contiguous bytes.\n", cbSize));
-        return NULL;
-    }
-
-    AssertRelease(physAddr < _4G && physAddr + cbSize <= _4G);
-
-    pChunk->u32Signature     = VBGL_PH_CHUNKSIGNATURE;
-    pChunk->cbSize           = cbSize;
-    pChunk->physAddr         = (uint32_t)physAddr;
-    pChunk->cAllocatedBlocks = 0;
-    pChunk->pNext            = g_vbgldata.pChunkHead;
-    pChunk->pPrev            = NULL;
-
-    /* Initialize the free block, which now occupies entire chunk. */
-    pBlock = (VBGLPHYSHEAPBLOCK *)((char *)pChunk + sizeof (VBGLPHYSHEAPCHUNK));
-
-    vbglPhysHeapInitBlock (pBlock, pChunk, cbSize - sizeof (VBGLPHYSHEAPCHUNK) - sizeof (VBGLPHYSHEAPBLOCK));
-
-    vbglPhysHeapInsertBlock (NULL, pBlock);
-
-    g_vbgldata.pChunkHead = pChunk;
-
-    VBGL_PH_dprintf(("Allocated chunk %p, block = %p size=%x\n", pChunk, pBlock, cbSize));
-
-    return pBlock;
+    LogRel(("vbglPhysHeapChunkAlloc: failed to alloc %u (%#x) contiguous bytes.\n", cbChunk, cbChunk));
+    return NULL;
 }
 
 
-void vbglPhysHeapChunkDelete (VBGLPHYSHEAPCHUNK *pChunk)
+static void vbglPhysHeapChunkDelete (VBGLPHYSHEAPCHUNK *pChunk)
 {
     char *p;
     VBGL_PH_ASSERT(pChunk != NULL);
@@ -399,49 +400,75 @@ void vbglPhysHeapChunkDelete (VBGLPHYSHEAPCHUNK *pChunk)
 
 DECLR0VBGL(void *) VbglR0PhysHeapAlloc (uint32_t cbSize)
 {
-    VBGLPHYSHEAPBLOCK *pBlock, *iter;
-    int rc = vbglPhysHeapEnter ();
+    VBGLPHYSHEAPBLOCK *pBlock, *pIter;
+    int rc;
 
+    /*
+     * Align the size to a pointer size to avoid getting misaligned header pointers and whatnot.
+     */
+    cbSize = RT_ALIGN_32(cbSize, sizeof(void *));
+
+    rc = vbglPhysHeapEnter ();
     if (RT_FAILURE(rc))
         return NULL;
 
     dumpheap ("pre alloc");
 
-    pBlock = NULL;
-
-    /* If there are free blocks in the heap, look at them. */
-    iter = g_vbgldata.pFreeBlocksHead;
-
-    /* There will be not many blocks in the heap, so
-     * linear search would be fast enough.
+    /*
+     * Search the free list.  We do this in linear fashion as we don't expect
+     * there to be many blocks in the heap.
      */
 
-    while (iter)
+    pBlock = NULL;
+    if (cbSize <= PAGE_SIZE / 4 * 3)
     {
-        if (iter->cbDataSize == cbSize)
-        {
-            /* exact match */
-            pBlock = iter;
-            break;
-        }
+        /* Smaller than 3/4 page:  Prefer a free block that can keep the request within a single page,
+           so HGCM processing in VMMDev can use page locks instead of several reads and writes. */
 
-        /* Looking for a free block with nearest size */
-        if (iter->cbDataSize > cbSize)
-        {
-            if (pBlock)
+        VBGLPHYSHEAPBLOCK *pFallback = NULL;
+        for (pIter = g_vbgldata.pFreeBlocksHead; pIter != NULL; pIter = pIter->pNext)
+            if (pIter->cbDataSize >= cbSize)
             {
-                if (iter->cbDataSize < pBlock->cbDataSize)
+                if (pIter->cbDataSize == cbSize)
                 {
-                    pBlock = iter;
+                    if (PAGE_SIZE - ((uintptr_t)vbglPhysHeapBlock2Data(pIter) & PAGE_OFFSET_MASK) >= cbSize)
+                    {
+                        pBlock = pIter;
+                        break;
+                    }
+                    pFallback = pIter;
+                }
+                else
+                {
+                    if (!pFallback || pIter->cbDataSize < pFallback->cbDataSize)
+                        pFallback = pIter;
+                    if (PAGE_SIZE - ((uintptr_t)vbglPhysHeapBlock2Data(pIter) & PAGE_OFFSET_MASK) >= cbSize)
+                        if (!pBlock || pIter->cbDataSize < pBlock->cbDataSize)
+                            pBlock = pIter;
                 }
             }
-            else
-            {
-                pBlock = iter;
-            }
-        }
 
-        iter = iter->pNext;
+        if (!pBlock)
+            pBlock = pFallback;
+    }
+    else
+    {
+        /* Large than 3/4 page:  Find smallest free list match. */
+
+        for (pIter = g_vbgldata.pFreeBlocksHead; pIter != NULL; pIter = pIter->pNext)
+            if (pIter->cbDataSize >= cbSize)
+            {
+                if (pIter->cbDataSize == cbSize)
+                {
+                    /* Exact match - we're done! */
+                    pBlock = pIter;
+                    break;
+                }
+
+                /* Looking for a free block with nearest size. */
+                if (!pBlock || pIter->cbDataSize < pBlock->cbDataSize)
+                    pBlock = pIter;
+            }
     }
 
     if (!pBlock)
@@ -465,17 +492,17 @@ DECLR0VBGL(void *) VbglR0PhysHeapAlloc (uint32_t cbSize)
         if (pBlock->cbDataSize > 2*(cbSize + sizeof (VBGLPHYSHEAPBLOCK)))
         {
             /* Data will occupy less than a half of the block,
-             * the block should be split.
+             * split off the tail end into a new free list entry.
              */
-            iter = (VBGLPHYSHEAPBLOCK *)((char *)pBlock + sizeof (VBGLPHYSHEAPBLOCK) + cbSize);
+            pIter = (VBGLPHYSHEAPBLOCK *)((char *)pBlock + sizeof (VBGLPHYSHEAPBLOCK) + cbSize);
 
-            /* Init the new 'iter' block, initialized blocks are always marked as free. */
-            vbglPhysHeapInitBlock (iter, pBlock->pChunk, pBlock->cbDataSize - cbSize - sizeof (VBGLPHYSHEAPBLOCK));
+            /* Init the new 'pIter' block, initialized blocks are always marked as free. */
+            vbglPhysHeapInitBlock (pIter, pBlock->pChunk, pBlock->cbDataSize - cbSize - sizeof (VBGLPHYSHEAPBLOCK));
 
             pBlock->cbDataSize = cbSize;
 
-            /* Insert the new 'iter' block after the 'pBlock' in the free list */
-            vbglPhysHeapInsertBlock (pBlock, iter);
+            /* Insert the new 'pIter' block after the 'pBlock' in the free list */
+            vbglPhysHeapInsertBlock (pBlock, pIter);
         }
 
         /* Exclude pBlock from free list */
