@@ -245,6 +245,10 @@ static int vbox_crtc_set_base(struct drm_crtc *crtc,
 			vbox_bo_unpin(bo);
 			vbox_bo_unreserve(bo);
 		}
+		else
+		{
+			DRM_ERROR("unable to lock buffer object: error %d\n", ret);
+		}
 	}
 
 	if (&vbox->fbdev->afb == vbox_fb)
@@ -398,7 +402,7 @@ static struct drm_encoder *drm_encoder_find(struct drm_device *dev, u32 id)
 static struct drm_encoder *vbox_best_single_encoder(struct drm_connector
 						    *connector)
 {
-#if RTLNX_VER_MIN(5,5,0) || RTLNX_RHEL_MIN(8,3)
+#if RTLNX_VER_MIN(5,5,0) || RTLNX_RHEL_MIN(8,3) || RTLNX_SUSE_MAJ_PREREQ(15,3)
         struct drm_encoder *encoder;
 
         /* There is only one encoder per connector */
@@ -856,7 +860,13 @@ static int vbox_cursor_set2(struct drm_crtc *crtc, struct drm_file *file_priv,
 	vbox->cursor_data_size = data_size;
 	dst = vbox->cursor_data;
 
+#if RTLNX_VER_MIN(5,14,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
+	ret = ttm_bo_kmap(&bo->bo, 0, bo->bo.resource->num_pages, &uobj_map);
+#elif RTLNX_VER_MIN(5,12,0) || RTLNX_RHEL_MAJ_PREREQ(8,5)
+	ret = ttm_bo_kmap(&bo->bo, 0, bo->bo.mem.num_pages, &uobj_map);
+#else
 	ret = ttm_bo_kmap(&bo->bo, 0, bo->bo.num_pages, &uobj_map);
+#endif
 	if (ret) {
 		vbox->cursor_data_size = 0;
 		goto out_unreserve_bo;
@@ -884,7 +894,7 @@ out_unmap_bo:
 out_unreserve_bo:
 	vbox_bo_unreserve(bo);
 out_unref_obj:
-#if RTLNX_VER_MIN(5,9,0)
+#if RTLNX_VER_MIN(5,9,0) || RTLNX_RHEL_MIN(8,4) || RTLNX_SUSE_MAJ_PREREQ(15,3)
 	drm_gem_object_put(obj);
 #else
 	drm_gem_object_put_unlocked(obj);
