@@ -59,6 +59,8 @@
 #include <iprt/path.h>
 #include <iprt/string.h>
 #include <iprt/thread.h>
+#include <iprt/env.h>
+#include <iprt/process.h>
 
 #include <X11/Xlibint.h>
 #include <X11/extensions/Xrandr.h>
@@ -203,7 +205,7 @@ static int determineOutputCount();
 #define checkFunctionPtr(pFunction) \
     do { \
         if (!pFunction) \
-            VBClLogFatalError("Could not find symbol address (%s)\n", #pFunction);\
+            VBClLogError("Could not find symbol address (%s)\n", #pFunction);\
     } while (0)
 
 
@@ -771,11 +773,25 @@ static bool startDRMClient()
     VBClLogInfo("Starting DRM client.\n");
     int rc = execve(szDRMClientPath, argv, env);
     if (rc == -1)
-        VBClLogFatalError("execve for % returns the following error %d %s\n", szDRMClientPath, errno, strerror(errno));
+        VBClLogFatalError("execve for %s returns the following error %d %s\n", szDRMClientPath, errno, strerror(errno));
     /* This is reached only when execve fails. */
     return false;
 }
 
+static bool legacyX11AgentStart()
+{
+#if defined(RT_OS_LINUX)
+# define VBOX_DRMCLIENT_LEGACY_EXECUTABLE    "/usr/bin/VBoxClient"
+    const char *apszArgs[3] = { VBOX_DRMCLIENT_LEGACY_EXECUTABLE, "--display", NULL };
+
+    int rc = RTProcCreate(VBOX_DRMCLIENT_LEGACY_EXECUTABLE, apszArgs, RTENV_DEFAULT,
+                          RTPROC_FLAGS_DETACHED | RTPROC_FLAGS_SEARCH_PATH, NULL);
+    return RT_SUCCESS(rc);
+#else
+    return false;
+#endif
+}
+
 static bool init()
 {
     /* If DRM client is already running don't start this service. */
@@ -857,43 +873,43 @@ static int openLibRandR()
     x11Context.fMonitorInfoAvailable = x11Context.pXRRGetMonitors && x11Context.pXRRFreeMonitors;
 
     *(void **)(&x11Context.pXRRGetScreenResources) = dlsym(x11Context.pRandLibraryHandle, "XRRGetScreenResources");
-    checkFunctionPtrReturn(x11Context.pXRRGetScreenResources);
+    checkFunctionPtr(x11Context.pXRRGetScreenResources);
 
     *(void **)(&x11Context.pXRRSetCrtcConfig) = dlsym(x11Context.pRandLibraryHandle, "XRRSetCrtcConfig");
-    checkFunctionPtrReturn(x11Context.pXRRSetCrtcConfig);
+    checkFunctionPtr(x11Context.pXRRSetCrtcConfig);
 
     *(void **)(&x11Context.pXRRFreeScreenResources) = dlsym(x11Context.pRandLibraryHandle, "XRRFreeScreenResources");
-    checkFunctionPtrReturn(x11Context.pXRRFreeScreenResources);
+    checkFunctionPtr(x11Context.pXRRFreeScreenResources);
 
     *(void **)(&x11Context.pXRRFreeModeInfo) = dlsym(x11Context.pRandLibraryHandle, "XRRFreeModeInfo");
-    checkFunctionPtrReturn(x11Context.pXRRFreeModeInfo);
+    checkFunctionPtr(x11Context.pXRRFreeModeInfo);
 
     *(void **)(&x11Context.pXRRFreeOutputInfo) = dlsym(x11Context.pRandLibraryHandle, "XRRFreeOutputInfo");
-    checkFunctionPtrReturn(x11Context.pXRRFreeOutputInfo);
+    checkFunctionPtr(x11Context.pXRRFreeOutputInfo);
 
     *(void **)(&x11Context.pXRRSetScreenSize) = dlsym(x11Context.pRandLibraryHandle, "XRRSetScreenSize");
-    checkFunctionPtrReturn(x11Context.pXRRSetScreenSize);
+    checkFunctionPtr(x11Context.pXRRSetScreenSize);
 
     *(void **)(&x11Context.pXRRUpdateConfiguration) = dlsym(x11Context.pRandLibraryHandle, "XRRUpdateConfiguration");
-    checkFunctionPtrReturn(x11Context.pXRRUpdateConfiguration);
+    checkFunctionPtr(x11Context.pXRRUpdateConfiguration);
 
     *(void **)(&x11Context.pXRRAllocModeInfo) = dlsym(x11Context.pRandLibraryHandle, "XRRAllocModeInfo");
-    checkFunctionPtrReturn(x11Context.pXRRAllocModeInfo);
+    checkFunctionPtr(x11Context.pXRRAllocModeInfo);
 
     *(void **)(&x11Context.pXRRCreateMode) = dlsym(x11Context.pRandLibraryHandle, "XRRCreateMode");
-    checkFunctionPtrReturn(x11Context.pXRRCreateMode);
+    checkFunctionPtr(x11Context.pXRRCreateMode);
 
     *(void **)(&x11Context.pXRRGetOutputInfo) = dlsym(x11Context.pRandLibraryHandle, "XRRGetOutputInfo");
-    checkFunctionPtrReturn(x11Context.pXRRGetOutputInfo);
+    checkFunctionPtr(x11Context.pXRRGetOutputInfo);
 
     *(void **)(&x11Context.pXRRGetCrtcInfo) = dlsym(x11Context.pRandLibraryHandle, "XRRGetCrtcInfo");
-    checkFunctionPtrReturn(x11Context.pXRRGetCrtcInfo);
+    checkFunctionPtr(x11Context.pXRRGetCrtcInfo);
 
     *(void **)(&x11Context.pXRRFreeCrtcInfo) = dlsym(x11Context.pRandLibraryHandle, "XRRFreeCrtcInfo");
-    checkFunctionPtrReturn(x11Context.pXRRFreeCrtcInfo);
+    checkFunctionPtr(x11Context.pXRRFreeCrtcInfo);
 
     *(void **)(&x11Context.pXRRAddOutputMode) = dlsym(x11Context.pRandLibraryHandle, "XRRAddOutputMode");
-    checkFunctionPtrReturn(x11Context.pXRRAddOutputMode);
+    checkFunctionPtr(x11Context.pXRRAddOutputMode);
 
     return VINF_SUCCESS;
 }
@@ -976,11 +992,17 @@ static void x11Connect()
         }
         if (x11Context.hRandRMajor < 1 || x11Context.hRandRMinor <= 3)
         {
-            VBClLogFatalError("Resizing service requires libXrandr Version >= 1.4. Detected version is %d.%d\n", x11Context.hRandRMajor, x11Context.hRandRMinor);
+            VBClLogError("Resizing service requires libXrandr Version >= 1.4. Detected version is %d.%d\n", x11Context.hRandRMajor, x11Context.hRandRMinor);
             XCloseDisplay(x11Context.pDisplay);
             x11Context.pDisplay = NULL;
+
+            bool rc = legacyX11AgentStart();
+            VBClLogInfo("Attempt to start legacy X11 resize agent has %s\n", rc ? "succeeded" : "failed");
+
             return;
         }
+        else
+            VBClLogInfo("Found libXrandr %d.%d\n", x11Context.hRandRMajor, x11Context.hRandRMinor);
     }
     x11Context.rootWindow = DefaultRootWindow(x11Context.pDisplay);
     x11Context.hEventMask = RRScreenChangeNotifyMask;
@@ -1274,6 +1296,12 @@ static bool configureOutput(int iOutputIndex, struct RANDROUTPUT *paOutputs)
 /** Construct the xrandr command which sets the whole monitor topology each time. */
 static void setXrandrTopology(struct RANDROUTPUT *paOutputs)
 {
+    if (!x11Context.pDisplay)
+    {
+        VBClLogInfo("not connected to X11\n");
+        return;
+    }
+
     XGrabServer(x11Context.pDisplay);
     if (x11Context.fWmwareCtrlExtention)
         callVMWCTRL(paOutputs);
@@ -1412,9 +1440,9 @@ static int run(struct VBCLSERVICE **ppInterface, bool fDaemonised)
         rc = VbglR3GetDisplayChangeRequestMulti(VMW_MAX_HEADS, &cDisplaysOut, aDisplays, fAck);
         fAck = true;
         if (RT_FAILURE(rc))
-            VBClLogFatalError("Failed to get display change request, rc=%Rrc\n", rc);
+            VBClLogError("Failed to get display change request, rc=%Rrc\n", rc);
         if (cDisplaysOut > VMW_MAX_HEADS)
-            VBClLogFatalError("Display change request contained, rc=%Rrc\n", rc);
+            VBClLogError("Display change request contained, rc=%Rrc\n", rc);
         if (cDisplaysOut > 0)
         {
             for (unsigned i = 0; i < cDisplaysOut && i < VMW_MAX_HEADS; ++i)
