@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2017 Oracle Corporation
+ * Copyright (C) 2008-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,6 +24,7 @@
 #include "DBGPlugInCommonELF.h"
 #include <VBox/vmm/dbgf.h>
 #include <iprt/asm.h>
+#include <iprt/err.h>
 #include <iprt/mem.h>
 #include <iprt/stream.h>
 #include <iprt/string.h>
@@ -353,6 +354,18 @@ static DECLCALLBACK(int)  dbgDiggerSolarisInit(PUVM pUVM, void *pvData);
 
 
 
+/**
+ * @copydoc DBGFOSREG::pfnStackUnwindAssist
+ */
+static DECLCALLBACK(int) dbgDiggerSolarisStackUnwindAssist(PUVM pUVM, void *pvData, VMCPUID idCpu, PDBGFSTACKFRAME pFrame,
+                                                           PRTDBGUNWINDSTATE pState, PCCPUMCTX pInitialCtx, RTDBGAS hAs,
+                                                           uint64_t *puScratch)
+{
+    RT_NOREF(pUVM, pvData, idCpu, pFrame, pState, pInitialCtx, hAs, puScratch);
+    return VINF_SUCCESS;
+}
+
+
 /**
  * @copydoc DBGFOSREG::pfnQueryInterface
  */
@@ -1110,18 +1123,19 @@ static DECLCALLBACK(int)  dbgDiggerSolarisConstruct(PUVM pUVM, void *pvData)
 
 const DBGFOSREG g_DBGDiggerSolaris =
 {
-    /* .u32Magic = */           DBGFOSREG_MAGIC,
-    /* .fFlags = */             0,
-    /* .cbData = */             sizeof(DBGDIGGERSOLARIS),
-    /* .szName = */             "Solaris",
-    /* .pfnConstruct = */       dbgDiggerSolarisConstruct,
-    /* .pfnDestruct = */        dbgDiggerSolarisDestruct,
-    /* .pfnProbe = */           dbgDiggerSolarisProbe,
-    /* .pfnInit = */            dbgDiggerSolarisInit,
-    /* .pfnRefresh = */         dbgDiggerSolarisRefresh,
-    /* .pfnTerm = */            dbgDiggerSolarisTerm,
-    /* .pfnQueryVersion = */    dbgDiggerSolarisQueryVersion,
-    /* .pfnQueryInterface = */  dbgDiggerSolarisQueryInterface,
-    /* .u32EndMagic = */        DBGFOSREG_MAGIC
+    /* .u32Magic = */               DBGFOSREG_MAGIC,
+    /* .fFlags = */                 0,
+    /* .cbData = */                 sizeof(DBGDIGGERSOLARIS),
+    /* .szName = */                 "Solaris",
+    /* .pfnConstruct = */           dbgDiggerSolarisConstruct,
+    /* .pfnDestruct = */            dbgDiggerSolarisDestruct,
+    /* .pfnProbe = */               dbgDiggerSolarisProbe,
+    /* .pfnInit = */                dbgDiggerSolarisInit,
+    /* .pfnRefresh = */             dbgDiggerSolarisRefresh,
+    /* .pfnTerm = */                dbgDiggerSolarisTerm,
+    /* .pfnQueryVersion = */        dbgDiggerSolarisQueryVersion,
+    /* .pfnQueryInterface = */      dbgDiggerSolarisQueryInterface,
+    /* .pfnStackUnwindAssist = */   dbgDiggerSolarisStackUnwindAssist,
+    /* .u32EndMagic = */            DBGFOSREG_MAGIC
 };
 
