@@ -349,12 +349,10 @@ typedef struct VGAState
     /** Current refresh timer interval. */
     uint32_t                    cMilliesRefreshInterval;
     /** Bitmap tracking dirty pages. */
-    uint32_t                    au32DirtyBitmap[VGA_VRAM_MAX / PAGE_SIZE / 32];
+    uint64_t                    bmDirtyBitmap[VGA_VRAM_MAX / PAGE_SIZE / 64];
 
     /** Flag indicating that there are dirty bits. This is used to optimize the handler resetting. */
     bool                        fHasDirtyBits;
-    /** LFB was updated flag. */
-    bool                        fLFBUpdated;
     /** Flag indicating that the VGA memory in the 0xa0000-0xbffff region has been remapped to allow direct access. */
     bool                        fRemappedVGA;
     /** Whether to render the guest VRAM to the framebuffer memory. False only for some LFB modes. */
@@ -368,9 +366,8 @@ typedef struct VGAState
     bool                        fVMSVGAEnabled;
     bool                        fVMSVGAPciId;
     bool                        fVMSVGAPciBarLayout;
-    bool                        Padding4[3];
 #else
-    bool                        Padding4[3+3];
+    bool                        afPadding4[3];
 #endif
 
     struct {
@@ -382,9 +379,6 @@ typedef struct VGAState
 #endif
     } pciRegions;
 
-    /** Physical access type for the linear frame buffer dirty page tracking. */
-    PGMPHYSHANDLERTYPE          hLfbAccessHandlerType;
-
     /** The physical address the VRAM was assigned. */
     RTGCPHYS                    GCPhysVRAM;
     /** The critical section protect the instance data. */
