@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,9 +22,11 @@
 #define LOG_GROUP LOG_GROUP_DIS
 #include <VBox/dis.h>
 #include <VBox/disopcode.h>
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 #include <VBox/log.h>
-#include <VBox/vmm/cpum.h>
+#ifdef RT_ARCH_AMD64
+# include <VBox/vmm/cpum.h>
+#endif
 #include <iprt/assert.h>
 #include <iprt/string.h>
 #include <iprt/stdarg.h>
@@ -34,6 +36,7 @@
 /*********************************************************************************************************************************
 *   Global Variables                                                                                                             *
 *********************************************************************************************************************************/
+#ifdef RT_ARCH_AMD64
 
 /**
  * Array for accessing 64-bit general registers in VMMREGFRAME structure
@@ -62,9 +65,9 @@ static const unsigned g_aReg64Index[] =
 /**
  * Macro for accessing 64-bit general purpose registers in CPUMCTXCORE structure.
  */
-#define DIS_READ_REG64(p, idx)       (*(uint64_t *)((char *)(p) + g_aReg64Index[idx]))
-#define DIS_WRITE_REG64(p, idx, val) (*(uint64_t *)((char *)(p) + g_aReg64Index[idx]) = val)
-#define DIS_PTR_REG64(p, idx)        ( (uint64_t *)((char *)(p) + g_aReg64Index[idx]))
+# define DIS_READ_REG64(p, idx)       (*(uint64_t *)((char *)(p) + g_aReg64Index[idx]))
+# define DIS_WRITE_REG64(p, idx, val) (*(uint64_t *)((char *)(p) + g_aReg64Index[idx]) = val)
+# define DIS_PTR_REG64(p, idx)        ( (uint64_t *)((char *)(p) + g_aReg64Index[idx]))
 
 /**
  * Array for accessing 32-bit general registers in VMMREGFRAME structure
@@ -93,14 +96,14 @@ static const unsigned g_aReg32Index[] =
 /**
  * Macro for accessing 32-bit general purpose registers in CPUMCTXCORE structure.
  */
-#define DIS_READ_REG32(p, idx)       (*(uint32_t *)((char *)(p) + g_aReg32Index[idx]))
+# define DIS_READ_REG32(p, idx)       (*(uint32_t *)((char *)(p) + g_aReg32Index[idx]))
 /* From http://www.cs.cmu.edu/~fp/courses/15213-s06/misc/asm64-handout.pdf:
  * ``Perhaps unexpectedly, instructions that move or generate 32-bit register
  *   values also set the upper 32 bits of the register to zero. Consequently
  *   there is no need for an instruction movzlq.''
  */
-#define DIS_WRITE_REG32(p, idx, val) (*(uint64_t *)((char *)(p) + g_aReg32Index[idx]) = (uint32_t)val)
-#define DIS_PTR_REG32(p, idx)        ( (uint32_t *)((char *)(p) + g_aReg32Index[idx]))
+# define DIS_WRITE_REG32(p, idx, val) (*(uint64_t *)((char *)(p) + g_aReg32Index[idx]) = (uint32_t)val)
+# define DIS_PTR_REG32(p, idx)        ( (uint32_t *)((char *)(p) + g_aReg32Index[idx]))
 
 /**
  * Array for accessing 16-bit general registers in CPUMCTXCORE structure
@@ -129,9 +132,9 @@ static const unsigned g_aReg16Index[] =
 /**
  * Macro for accessing 16-bit general purpose registers in CPUMCTXCORE structure.
  */
-#define DIS_READ_REG16(p, idx)          (*(uint16_t *)((char *)(p) + g_aReg16Index[idx]))
-#define DIS_WRITE_REG16(p, idx, val)    (*(uint16_t *)((char *)(p) + g_aReg16Index[idx]) = val)
-#define DIS_PTR_REG16(p, idx)           ( (uint16_t *)((char *)(p) + g_aReg16Index[idx]))
+# define DIS_READ_REG16(p, idx)          (*(uint16_t *)((char *)(p) + g_aReg16Index[idx]))
+# define DIS_WRITE_REG16(p, idx, val)    (*(uint16_t *)((char *)(p) + g_aReg16Index[idx]) = val)
+# define DIS_PTR_REG16(p, idx)           ( (uint16_t *)((char *)(p) + g_aReg16Index[idx]))
 
 /**
  * Array for accessing 8-bit general registers in CPUMCTXCORE structure
@@ -164,9 +167,9 @@ static const unsigned g_aReg8Index[] =
 /**
  * Macro for accessing 8-bit general purpose registers in CPUMCTXCORE structure.
  */
-#define DIS_READ_REG8(p, idx)           (*(uint8_t *)((char *)(p) + g_aReg8Index[idx]))
-#define DIS_WRITE_REG8(p, idx, val)     (*(uint8_t *)((char *)(p) + g_aReg8Index[idx]) = val)
-#define DIS_PTR_REG8(p, idx)            ( (uint8_t *)((char *)(p) + g_aReg8Index[idx]))
+# define DIS_READ_REG8(p, idx)           (*(uint8_t *)((char *)(p) + g_aReg8Index[idx]))
+# define DIS_WRITE_REG8(p, idx, val)     (*(uint8_t *)((char *)(p) + g_aReg8Index[idx]) = val)
+# define DIS_PTR_REG8(p, idx)            ( (uint8_t *)((char *)(p) + g_aReg8Index[idx]))
 
 /**
  * Array for accessing segment registers in CPUMCTXCORE structure
@@ -195,8 +198,11 @@ static const unsigned g_aRegHidSegIndex[] =
 /**
  * Macro for accessing segment registers in CPUMCTXCORE structure.
  */
-#define DIS_READ_REGSEG(p, idx)         (*((uint16_t *)((char *)(p) + g_aRegSegIndex[idx])))
-#define DIS_WRITE_REGSEG(p, idx, val)   (*((uint16_t *)((char *)(p) + g_aRegSegIndex[idx])) = val)
+# define DIS_READ_REGSEG(p, idx)         (*((uint16_t *)((char *)(p) + g_aRegSegIndex[idx])))
+# define DIS_WRITE_REGSEG(p, idx, val)   (*((uint16_t *)((char *)(p) + g_aRegSegIndex[idx])) = val)
+
+#endif /* RT_ARCH_AMD64 */
+
 
 //*****************************************************************************
 //*****************************************************************************
@@ -333,6 +339,8 @@ DISDECL(uint8_t) DISQuerySegPrefixByte(PCDISSTATE pDis)
 }
 
 
+#ifdef RT_ARCH_AMD64
+
 /**
  * Returns the value of the specified 8 bits general purpose register
  *
@@ -834,3 +842,5 @@ DISDECL(int) DISQueryParamRegPtr(PCPUMCTXCORE pCtx, PCDISSTATE pDis, PCDISOPPARA
     return VERR_INVALID_PARAMETER;
 }
 
+#endif /* RT_ARCH_AMD64 */
+
