@@ -790,7 +790,6 @@ void printUsage(USAGECATEGORY enmCommand, uint64_t fSubcommandScope, PRTSTREAM p
                            "%s export %s          <machines> --output|-o <name>.<ovf/ova/tar.gz>\n"
                      "                            [--legacy09|--ovf09|--ovf10|--ovf20|--opc10]\n"
                      "                            [--manifest]\n"
-                     "                            [--iso]\n"
                      "                            [--options manifest|iso|nomacs|nomacsbutnat]\n"
                      "                            [--vsys <number of virtual system>]\n"
                      "                                    [--vmname <name>]\n"
@@ -816,6 +815,7 @@ void printUsage(USAGECATEGORY enmCommand, uint64_t fSubcommandScope, PRTSTREAM p
                      "                                    [--cloudocisubnet <OCI subnet id>]\n"
                      "                                    [--cloudpublicip <true/false>]\n"
                      "                                    [--cloudprivateip <ip>]\n"
+                     "                                    [--cloudinitscriptpath <script path>]\n"
                      "\n", SEP);
 
     if (enmCommand == USAGE_STARTVM || enmCommand == USAGE_S_ALL)
@@ -1214,11 +1214,14 @@ void printUsage(USAGECATEGORY enmCommand, uint64_t fSubcommandScope, PRTSTREAM p
         RTStrmPrintf(pStrm,
                            "%s hostonlyif %s      ipconfig <name>\n"
                      "                            [--dhcp |\n"
-                     "                            --ip<ipv4> [--netmask<ipv4> (def: 255.255.255.0)] |\n"
-                     "                            --ipv6<ipv6> [--netmasklengthv6<length> (def: 64)]]\n"
+                     "                            --ip <ipv4> [--netmask <ipv4> (def:255.255.255.0)]|\n"
+                     "                            --ipv6 <ipv6> [--netmasklengthv6 <N> (def:64)]]"
 # if !defined(RT_OS_SOLARIS) || defined(VBOX_ONLY_DOCS)
+                     " |\n"
                      "                            create |\n"
                      "                            remove <name>\n"
+# else
+                     "\n"
 # endif
                      "\n", SEP);
     }
