@@ -1980,15 +1980,12 @@ static bool vboxdrvDarwinCpuHasSMAP(void)
 }
 
 
-RTDECL(int) SUPR0Printf(const char *pszFormat, ...)
+RTDECL(int) SUPR0PrintfV(const char *pszFormat, va_list va)
 {
     IPRT_DARWIN_SAVE_EFL_AC();
-    va_list     va;
-    char        szMsg[512];
 
-    va_start(va, pszFormat);
+    char szMsg[512];
     RTStrPrintfV(szMsg, sizeof(szMsg) - 1, pszFormat, va);
-    va_end(va);
     szMsg[sizeof(szMsg) - 1] = '\0';
 
     printf("%s", szMsg);
