@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008-2017 Oracle Corporation
+ * Copyright (C) 2008-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -433,14 +433,14 @@ LOCAL int VBoxNetFltSolarisGetInfo(dev_info_t *pDip, ddi_info_cmd_t enmCmd, void
         case DDI_INFO_DEVT2DEVINFO:
         {
             *ppvResult = g_pVBoxNetFltSolarisDip;
-            return DDI_SUCCESS;
+            return *ppvResult ? DDI_SUCCESS : DDI_FAILURE;
         }
 
         case DDI_INFO_DEVT2INSTANCE:
         {
-            int instance = getminor((dev_t)pvArg);
-            *ppvResult = (void *)(uintptr_t)instance;
-            return DDI_SUCCESS;
+            /* There can only be a single-instance of this driver and thus its instance number is 0. */
+            *ppvResult = (void *)0;
+            break;
         }
     }
 
