@@ -1293,7 +1293,7 @@ static NTSTATUS vboxUsbMonIoctlDispatch(PVBOXUSBMONCTX pContext, ULONG Ctl, PVOI
 
         case SUPUSBFLT_IOCTL_GET_DEVICE:
         {
-            HVBOXUSBDEVUSR hDevice = *((HVBOXUSBDEVUSR*)pvBuffer);
+            HVBOXUSBDEVUSR hDevice;
             PUSBSUP_GETDEV_MON pOut = (PUSBSUP_GETDEV_MON)pvBuffer;
             if (!pvBuffer || cbInBuffer != sizeof (hDevice) || cbOutBuffer < sizeof (*pOut))
             {
@@ -1302,6 +1302,7 @@ static NTSTATUS vboxUsbMonIoctlDispatch(PVBOXUSBMONCTX pContext, ULONG Ctl, PVOI
                 Status = STATUS_INVALID_PARAMETER;
                 break;
             }
+            hDevice = *(HVBOXUSBDEVUSR*)pvBuffer;
             if (!hDevice)
             {
                 WARN(("SUPUSBFLT_IOCTL_GET_DEVICE: hDevice is NULL!",
