@@ -3,7 +3,7 @@
 # Linux kernel module init script
 
 #
-# Copyright (C) 2006-2017 Oracle Corporation
+# Copyright (C) 2006-2020 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -86,6 +86,27 @@ else
     DEVICE_MODE=0660
 fi
 
+KERN_VER=`uname -r`
+if test -e "${MODULE_SRC}/vboxpci"; then
+    MODULE_LIST="vboxdrv vboxnetflt vboxnetadp vboxpci"
+else
+    MODULE_LIST="vboxdrv vboxnetflt vboxnetadp"
+fi
+# Secure boot state.
+case "`mokutil --sb-state 2>/dev/null`" in
+    *"disabled in shim"*) unset HAVE_SEC_BOOT;;
+    *"SecureBoot enabled"*) HAVE_SEC_BOOT=true;;
+    *) unset HAVE_SEC_BOOT;;
+esac
+# So far we can only sign modules on Ubuntu and on Debian 10 and later.
+DEB_PUB_KEY=/var/lib/shim-signed/mok/MOK.der
+DEB_PRIV_KEY=/var/lib/shim-signed/mok/MOK.priv
+unset HAVE_DEB_KEY
+case "`mokutil --test-key "$DEB_PUB_KEY" 2>/dev/null`" in
+    *"is already"*) DEB_KEY_ENROLLED=true;;
+    *) unset DEB_KEY_ENROLLED;;
+esac
+
 [ -r /etc/default/virtualbox ] && . /etc/default/virtualbox
 
 # Preamble for Gentoo
@@ -135,6 +156,12 @@ module_build_log()
         >> "${LOG}"
 }
 
+# Detect VirtualBox version info or report error.
+VBOX_VERSION="`"$VBOXMANAGE" -v | cut -d r -f1`"
+[ -n "$VBOX_VERSION" ] || failure 'Cannot detect VirtualBox version number'
+VBOX_REVISION="r`"$VBOXMANAGE" -v | cut -d r -f2`"
+[ "$VBOX_REVISION" != "r" ] || failure 'Cannot detect VirtualBox revision number'
+
 ## Output the vboxdrv part of our udev rule.  This is redirected to the right file.
 udev_write_vboxdrv() {
     VBOXDRV_GRP="$1"
@@ -246,21 +273,102 @@ cleanup_usb()
     rm -rf /dev/vboxusb
 }
 
+# Returns path to module file as seen by modinfo(8) or empty string.
+module_path()
+{
+    mod="$1"
+    [ -n "$mod" ] || return
+
+    modinfo "$mod" 2>/dev/null | grep -e "^filename:" | tr -s ' ' | cut -d " " -f2
+}
+
+# Returns module version if module is available or empty string.
+module_version()
+{
+    mod="$1"
+    [ -n "$mod" ] || return
+
+    modinfo "$mod" 2>/dev/null | grep -e "^version:" | tr -s ' ' | cut -d " " -f2
+}
+
+# Returns module revision if module is available in the system or empty string.
+module_revision()
+{
+    mod="$1"
+    [ -n "$mod" ] || return
+
+    modinfo "$mod" 2>/dev/null | grep -e "^version:" | tr -s ' ' | cut -d " " -f3
+}
+
+# Returns "1" if externally built module is available in the system and its
+# version and revision number do match to current VirtualBox installation.
+# Or empty string otherwise.
+module_available()
+{
+    mod="$1"
+    [ -n "$mod" ] || return
+
+    [ "$VBOX_VERSION" = "$(module_version "$mod")" ] || return
+    [ "$VBOX_REVISION" = "$(module_revision "$mod")" ] || return
+
+    # Check if module belongs to VirtualBox installation.
+    #
+    # We have a convention that only modules from /lib/modules/*/misc
+    # belong to us. Modules from other locations are treated as
+    # externally built.
+    mod_path="$(module_path "$mod")"
+
+    # If module path points to a symbolic link, resolve actual file location.
+    [ -L "$mod_path" ] && mod_path="$(readlink -e -- "$mod_path")"
+
+    # File exists?
+    [ -f "$mod_path" ] || return
+
+    # Extract last component of module path and check whether it is located
+    # outside of /lib/modules/*/misc.
+    mod_dir="$(dirname "$mod_path" | sed 's;^.*/;;')"
+    [ "$mod_dir" = "misc" ] || return
+
+    echo "1"
+}
+
+# Check if required modules are installed in the system and versions match.
+setup_complete()
+{
+    [ "$(module_available vboxdrv)"    = "1" ] || return
+    [ "$(module_available vboxnetflt)" = "1" ] || return
+    [ "$(module_available vboxnetadp)" = "1" ] || return
+
+    # All modules are in place.
+    echo "1"
+}
+
 start()
 {
     begin_msg "Starting VirtualBox services" console
     if [ -d /proc/xen ]; then
         failure "Running VirtualBox in a Xen environment is not supported"
     fi
+    if test -n "$HAVE_SEC_BOOT" && test -z "$DEB_KEY_ENROLLED"; then
+        if test -n "$HAVE_DEB_KEY"; then
+            begin_msg "You must re-start your system to finish Debian secure boot set-up." console
+        else
+            begin_msg "You must sign these kernel modules before using VirtualBox:
+  $MODULE_LIST
+See the documentation for your Linux distribution." console
+        fi
+    fi
+
     if ! running vboxdrv; then
+
+        # Check if system already has matching modules installed.
+        [ "$(setup_complete)" = "1" ] || setup
+
         if ! rm -f $DEVICE; then
             failure "Cannot remove $DEVICE"
         fi
         if ! $MODPROBE vboxdrv > /dev/null 2>&1; then
-            setup
-            if ! $MODPROBE vboxdrv > /dev/null 2>&1; then
-                failure "modprobe vboxdrv failed. Please use 'dmesg' to find out why"
-            fi
+            failure "modprobe vboxdrv failed. Please use 'dmesg' to find out why"
         fi
         sleep .2
     fi
@@ -298,7 +406,7 @@ start()
     if ! $MODPROBE vboxnetadp > /dev/null 2>&1; then
         failure "modprobe vboxnetadp failed. Please use 'dmesg' to find out why"
     fi
-    if ! $MODPROBE vboxpci > /dev/null 2>&1; then
+    if test -e "${MODULE_SRC}/vboxpci" && ! $MODPROBE vboxpci > /dev/null 2>&1; then
         failure "modprobe vboxpci failed. Please use 'dmesg' to find out why"
     fi
     # Create the /dev/vboxusb directory if the host supports that method
@@ -409,6 +517,7 @@ cleanup()
                   "${i}/misc/vboxnetflt.ko" "${i}/misc/vboxpci.ko"
             version=`expr "${i}" : "/lib/modules/\(.*\)"`
             depmod -a "${version}"
+            sync
         fi
         # Remove the kernel version folder if it was empty except for us.
         test   "`echo ${i}/misc/* ${i}/misc/.?* ${i}/* ${i}/.?*`" \
@@ -449,18 +558,41 @@ setup()
         module_build_log "$myerr"
         failure "Look at $LOG to find out what went wrong"
     fi
-    log "Building the PCI pass-through module."
-    if ! myerr=`$BUILDINTMP \
-        --use-module-symvers /tmp/vboxdrv-Module.symvers \
-        --module-source "$MODULE_SRC/vboxpci" \
-        --no-print-directory install 2>&1`; then
-        log "Error building the module:"
-        module_build_log "$myerr"
-        failure "Look at $LOG to find out what went wrong"
+    if test -e "$MODULE_SRC/vboxpci"; then
+        log "Building the PCI pass-through module."
+        if ! myerr=`$BUILDINTMP \
+            --use-module-symvers /tmp/vboxdrv-Module.symvers \
+            --module-source "$MODULE_SRC/vboxpci" \
+            --no-print-directory install 2>&1`; then
+            log "Error building the module:"
+            module_build_log "$myerr"
+            failure "Look at $LOG to find out what went wrong"
+        fi
     fi
     rm -f /etc/vbox/module_not_compiled
     depmod -a
+    sync
     succ_msg "VirtualBox kernel modules built"
+    # Secure boot on Ubuntu and Debian.
+    if test -n "$HAVE_SEC_BOOT" &&
+        type update-secureboot-policy >/dev/null 2>&1; then
+        SHIM_NOTRIGGER=y update-secureboot-policy --new-key
+    fi
+    if test -f "$DEB_PUB_KEY" && test -f "$DEB_PRIV_KEY"; then
+        HAVE_DEB_KEY=true
+        for i in $MODULE_LIST; do
+            kmodsign sha512 /var/lib/shim-signed/mok/MOK.priv \
+                /var/lib/shim-signed/mok/MOK.der \
+                /lib/modules/"$KERN_VER"/misc/"$i".ko
+        done
+        # update-secureboot-policy "expects" DKMS modules.
+        # Work around this and talk to the authors as soon
+        # as possible to fix it.
+        mkdir -p /var/lib/dkms/vbox-temp
+        update-secureboot-policy --enroll-key 2>/dev/null ||
+            begin_msg "Failed to enroll secure boot key." console
+        rmdir -p /var/lib/dkms/vbox-temp 2>/dev/null
+    fi
 }
 
 dmnstatus()
@@ -533,7 +665,7 @@ status)
     dmnstatus
     ;;
 *)
-    echo "Usage: $0 {start|stop|stop_vms|restart|force-reload|status}"
+    echo "Usage: $0 {start|stop|stop_vms|restart|setup|cleanup|force-reload|status}"
     exit 1
 esac
 
