@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ____H_MACHINEIMPL
-#define ____H_MACHINEIMPL
+#ifndef MAIN_INCLUDED_MachineImpl_h
+#define MAIN_INCLUDED_MachineImpl_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include "AuthLibrary.h"
 #include "VirtualBoxBase.h"
@@ -31,6 +34,8 @@
 #include "SerialPortImpl.h"
 #include "ParallelPortImpl.h"
 #include "BIOSSettingsImpl.h"
+#include "RecordingSettingsImpl.h"
+#include "GraphicsAdapterImpl.h"
 #include "StorageControllerImpl.h"          // required for MachineImpl.h to compile on Windows
 #include "USBControllerImpl.h"              // required for MachineImpl.h to compile on Windows
 #include "BandwidthControlImpl.h"
@@ -46,8 +51,6 @@
 
 #include "VBox/com/ErrorInfo.h"
 
-#include <iprt/file.h>
-#include <iprt/thread.h>
 #include <iprt/time.h>
 
 #include <list>
@@ -194,7 +197,7 @@ public:
 
         // list of files to delete in Delete(); this list is filled by Unregister()
         std::list<Utf8Str>  llFilesToDelete;
-    };
+};
 
     /**
      *  Saved state data.
@@ -262,26 +265,14 @@ public:
         ULONG               mMemorySize;
         ULONG               mMemoryBalloonSize;
         BOOL                mPageFusionEnabled;
-        GraphicsControllerType_T mGraphicsControllerType;
-        ULONG               mVRAMSize;
-        ULONG               mVideoCaptureWidth;
-        ULONG               mVideoCaptureHeight;
-        ULONG               mVideoCaptureRate;
-        ULONG               mVideoCaptureFPS;
-        ULONG               mVideoCaptureMaxTime;
-        ULONG               mVideoCaptureMaxFileSize;
-        Utf8Str             mVideoCaptureOptions;
-        Utf8Str             mVideoCaptureFile;
-        BOOL                mVideoCaptureEnabled;
-        BOOL                maVideoCaptureScreens[SchemaDefs::MaxGuestMonitors];
-        ULONG               mMonitorCount;
+        settings::RecordingSettings mRecordSettings;
         BOOL                mHWVirtExEnabled;
         BOOL                mHWVirtExNestedPagingEnabled;
         BOOL                mHWVirtExLargePagesEnabled;
         BOOL                mHWVirtExVPIDEnabled;
         BOOL                mHWVirtExUXEnabled;
         BOOL                mHWVirtExForceEnabled;
-        BOOL                mAccelerate2DVideoEnabled;
+        BOOL                mHWVirtExUseNativeApi;
         BOOL                mPAEEnabled;
         settings::Hardware::LongModeType mLongMode;
         BOOL                mTripleFaultReset;
@@ -295,12 +286,12 @@ public:
         BOOL                mL1DFlushOnVMEntry;
         BOOL                mMDSClearOnSched;
         BOOL                mMDSClearOnVMEntry;
+        BOOL                mNestedHWVirt;
         ULONG               mCPUCount;
         BOOL                mCPUHotPlugEnabled;
         ULONG               mCpuExecutionCap;
         uint32_t            mCpuIdPortabilityLevel;
         Utf8Str             mCpuProfile;
-        BOOL                mAccelerate3DEnabled;
         BOOL                mHPETEnabled;
 
         BOOL                mCPUAttached[SchemaDefs::MaxCPUCount];
@@ -313,6 +304,8 @@ public:
         SharedFolderList    mSharedFolders;
 
         ClipboardMode_T     mClipboardMode;
+        BOOL                mClipboardFileTransfersEnabled;
+
         DnDMode_T           mDnDMode;
 
         typedef std::map<Utf8Str, GuestProperty> GuestPropertyMap;
@@ -352,6 +345,7 @@ public:
                  const Utf8Str &strConfigFile,
                  const Utf8Str &strName,
                  const StringsList &llGroups,
+                 const Utf8Str &strOsTypeId,
                  GuestOSType *aOsType,
                  const Guid &aId,
                  bool fForceOverwrite,
@@ -365,6 +359,7 @@ public:
     // initializer for machine config in memory (OVF import)
     HRESULT init(VirtualBox *aParent,
                  const Utf8Str &strName,
+                 const Utf8Str &strSettingsFilename,
                  const settings::MachineConfigFile &config);
 
     void uninit();
@@ -483,7 +478,9 @@ public:
         IsModified_BIOS                 = 0x0200,
         IsModified_SharedFolders        = 0x0400,
         IsModified_Snapshots            = 0x0800,
-        IsModified_BandwidthControl     = 0x1000
+        IsModified_BandwidthControl     = 0x1000,
+        IsModified_Recording            = 0x2000,
+        IsModified_GraphicsAdapter      = 0x4000,
     };
 
     /**
@@ -495,12 +492,15 @@ public:
      */
     Utf8Str i_getOSTypeId() const { return mUserData->s.strOsType; }
     ChipsetType_T i_getChipsetType() const { return mHWData->mChipsetType; }
+    FirmwareType_T i_getFirmwareType() const { return mHWData->mFirmwareType; }
     ParavirtProvider_T i_getParavirtProvider() const { return mHWData->mParavirtProvider; }
     Utf8Str i_getParavirtDebug() const { return mHWData->mParavirtDebug; }
 
     void i_setModified(uint32_t fl, bool fAllowStateModification = true);
     void i_setModifiedLock(uint32_t fl, bool fAllowStateModification = true);
 
+    MachineState_T i_getMachineState() const { return mData->mMachineState; }
+
     bool i_isStateModificationAllowed() const { return mData->m_fAllowStateModification; }
     void i_allowStateModification()           { mData->m_fAllowStateModification = true; }
     void i_disallowStateModification()        { mData->m_fAllowStateModification = false; }
@@ -517,17 +517,19 @@ public:
     virtual HRESULT i_onParallelPortChange(IParallelPort * /* parallelPort */) { return S_OK; }
     virtual HRESULT i_onVRDEServerChange(BOOL /* aRestart */) { return S_OK; }
     virtual HRESULT i_onUSBControllerChange() { return S_OK; }
-    virtual HRESULT i_onStorageControllerChange() { return S_OK; }
+    virtual HRESULT i_onStorageControllerChange(const com::Guid & /* aMachineId */, const com::Utf8Str & /* aControllerName */) { return S_OK; }
     virtual HRESULT i_onCPUChange(ULONG /* aCPU */, BOOL /* aRemove */) { return S_OK; }
     virtual HRESULT i_onCPUExecutionCapChange(ULONG /* aExecutionCap */) { return S_OK; }
     virtual HRESULT i_onMediumChange(IMediumAttachment * /* mediumAttachment */, BOOL /* force */) { return S_OK; }
     virtual HRESULT i_onSharedFolderChange() { return S_OK; }
+    virtual HRESULT i_onVMProcessPriorityChange(VMProcPriority_T /* aPriority */) { return S_OK; }
     virtual HRESULT i_onClipboardModeChange(ClipboardMode_T /* aClipboardMode */) { return S_OK; }
+    virtual HRESULT i_onClipboardFileTransferModeChange(BOOL /* aEnable */) { return S_OK; }
     virtual HRESULT i_onDnDModeChange(DnDMode_T /* aDnDMode */) { return S_OK; }
     virtual HRESULT i_onBandwidthGroupChange(IBandwidthGroup * /* aBandwidthGroup */) { return S_OK; }
     virtual HRESULT i_onStorageDeviceChange(IMediumAttachment * /* mediumAttachment */, BOOL /* remove */,
                                             BOOL /* silent */) { return S_OK; }
-    virtual HRESULT i_onVideoCaptureChange() { return S_OK; }
+    virtual HRESULT i_onRecordingChange(BOOL /* aEnable */) { return S_OK; }
 
     HRESULT i_saveRegistryEntry(settings::MachineRegistryEntry &data);
 
@@ -537,16 +539,16 @@ public:
     void i_getLogFolder(Utf8Str &aLogFolder);
     Utf8Str i_getLogFilename(ULONG idx);
     Utf8Str i_getHardeningLogFilename(void);
+    Utf8Str i_getDefaultNVRAMFilename();
+    Utf8Str i_getSnapshotNVRAMFilename();
 
     void i_composeSavedStateFilename(Utf8Str &strStateFilePath);
 
-    void i_getDefaultVideoCaptureFile(Utf8Str &strFile);
-
     bool i_isUSBControllerPresent();
 
     HRESULT i_launchVMProcess(IInternalSessionControl *aControl,
                               const Utf8Str &strType,
-                              const Utf8Str &strEnvironment,
+                              const std::vector<com::Utf8Str> &aEnvironmentChanges,
                               ProgressProxy *aProgress);
 
     HRESULT i_getDirectControl(ComPtr<IInternalSessionControl> *directControl)
@@ -580,7 +582,7 @@ public:
 
     HRESULT i_prepareRegister();
 
-    HRESULT i_getSharedFolder(CBSTR aName,
+    HRESULT i_getSharedFolder(const Utf8Str &aName,
                               ComObjPtr<SharedFolder> &aSharedFolder,
                               bool aSetError = false)
     {
@@ -623,7 +625,7 @@ protected:
 
     Machine *i_getMachine();
 
-    void i_ensureNoStateDependencies();
+    void i_ensureNoStateDependencies(AutoWriteLock &alock);
 
     virtual HRESULT i_setMachineState(MachineState_T aMachineState);
 
@@ -673,7 +675,7 @@ protected:
     };
 
     HRESULT i_prepareSaveSettings(bool *pfNeedsGlobalSaveSettings);
-    HRESULT i_saveSettings(bool *pfNeedsGlobalSaveSettings, int aFlags = 0);
+    HRESULT i_saveSettings(bool *pfNeedsGlobalSaveSettings, AutoWriteLock &alock, int aFlags = 0);
 
     void i_copyMachineDataToSettings(settings::MachineConfigFile &config);
     HRESULT i_saveAllSnapshots(settings::MachineConfigFile &config);
@@ -721,6 +723,8 @@ protected:
 
     Utf8Str i_getExtraData(const Utf8Str &strKey);
 
+    com::Utf8Str i_controllerNameFromBusType(StorageBus_T aBusType);
+
 #ifdef VBOX_WITH_GUEST_PROPS
     HRESULT i_getGuestPropertyFromService(const com::Utf8Str &aName, com::Utf8Str &aValue,
                                           LONG64 *aTimestamp, com::Utf8Str &aFlags) const;
@@ -782,6 +786,8 @@ protected:
     const ComObjPtr<AudioAdapter>      mAudioAdapter;
     const ComObjPtr<USBDeviceFilters>  mUSBDeviceFilters;
     const ComObjPtr<BIOSSettings>      mBIOSSettings;
+    const ComObjPtr<RecordingSettings> mRecordingSettings;
+    const ComObjPtr<GraphicsAdapter>   mGraphicsAdapter;
     const ComObjPtr<BandwidthControl>  mBandwidthControl;
 
     typedef std::vector<ComObjPtr<NetworkAdapter> > NetworkAdapterVector;
@@ -828,13 +834,15 @@ protected:
     class DeleteConfigTask;
     void i_deleteConfigHandler(DeleteConfigTask &task);
 
+    friend class Appliance;
+    friend class RecordingSettings;
+    friend class RecordingScreenSettings;
     friend class SessionMachine;
     friend class SnapshotMachine;
-    friend class Appliance;
     friend class VirtualBox;
 
     friend class MachineCloneVM;
-
+    friend class MachineMoveVM;
 private:
     // wrapped IMachine properties
     HRESULT getParent(ComPtr<IVirtualBox> &aParent);
@@ -871,37 +879,9 @@ private:
     HRESULT setMemoryBalloonSize(ULONG aMemoryBalloonSize);
     HRESULT getPageFusionEnabled(BOOL *aPageFusionEnabled);
     HRESULT setPageFusionEnabled(BOOL aPageFusionEnabled);
-    HRESULT getGraphicsControllerType(GraphicsControllerType_T *aGraphicsControllerType);
-    HRESULT setGraphicsControllerType(GraphicsControllerType_T aGraphicsControllerType);
-    HRESULT getVRAMSize(ULONG *aVRAMSize);
-    HRESULT setVRAMSize(ULONG aVRAMSize);
-    HRESULT getAccelerate3DEnabled(BOOL *aAccelerate3DEnabled);
-    HRESULT setAccelerate3DEnabled(BOOL aAccelerate3DEnabled);
-    HRESULT getAccelerate2DVideoEnabled(BOOL *aAccelerate2DVideoEnabled);
-    HRESULT setAccelerate2DVideoEnabled(BOOL aAccelerate2DVideoEnabled);
-    HRESULT getMonitorCount(ULONG *aMonitorCount);
-    HRESULT setMonitorCount(ULONG aMonitorCount);
-    HRESULT getVideoCaptureEnabled(BOOL *aVideoCaptureEnabled);
-    HRESULT setVideoCaptureEnabled(BOOL aVideoCaptureEnabled);
-    HRESULT getVideoCaptureScreens(std::vector<BOOL> &aVideoCaptureScreens);
-    HRESULT setVideoCaptureScreens(const std::vector<BOOL> &aVideoCaptureScreens);
-    HRESULT getVideoCaptureFile(com::Utf8Str &aVideoCaptureFile);
-    HRESULT setVideoCaptureFile(const com::Utf8Str &aVideoCaptureFile);
-    HRESULT getVideoCaptureWidth(ULONG *aVideoCaptureWidth);
-    HRESULT setVideoCaptureWidth(ULONG aVideoCaptureWidth);
-    HRESULT getVideoCaptureHeight(ULONG *aVideoCaptureHeight);
-    HRESULT setVideoCaptureHeight(ULONG aVideoCaptureHeight);
-    HRESULT getVideoCaptureRate(ULONG *aVideoCaptureRate);
-    HRESULT setVideoCaptureRate(ULONG aVideoCaptureRate);
-    HRESULT getVideoCaptureFPS(ULONG *aVideoCaptureFPS);
-    HRESULT setVideoCaptureFPS(ULONG aVideoCaptureFPS);
-    HRESULT getVideoCaptureMaxTime(ULONG *aVideoCaptureMaxTime);
-    HRESULT setVideoCaptureMaxTime(ULONG aVideoCaptureMaxTime);
-    HRESULT getVideoCaptureMaxFileSize(ULONG *aVideoCaptureMaxFileSize);
-    HRESULT setVideoCaptureMaxFileSize(ULONG aVideoCaptureMaxFileSize);
-    HRESULT getVideoCaptureOptions(com::Utf8Str &aVideoCaptureOptions);
-    HRESULT setVideoCaptureOptions(const com::Utf8Str &aVideoCaptureOptions);
+    HRESULT getGraphicsAdapter(ComPtr<IGraphicsAdapter> &aGraphicsAdapter);
     HRESULT getBIOSSettings(ComPtr<IBIOSSettings> &aBIOSSettings);
+    HRESULT getRecordingSettings(ComPtr<IRecordingSettings> &aRecordingSettings);
     HRESULT getFirmwareType(FirmwareType_T *aFirmwareType);
     HRESULT setFirmwareType(FirmwareType_T aFirmwareType);
     HRESULT getPointingHIDType(PointingHIDType_T *aPointingHIDType);
@@ -939,6 +919,8 @@ private:
     HRESULT getSharedFolders(std::vector<ComPtr<ISharedFolder> > &aSharedFolders);
     HRESULT getClipboardMode(ClipboardMode_T *aClipboardMode);
     HRESULT setClipboardMode(ClipboardMode_T aClipboardMode);
+    HRESULT getClipboardFileTransfersEnabled(BOOL *aEnabled);
+    HRESULT setClipboardFileTransfersEnabled(BOOL aEnabled);
     HRESULT getDnDMode(DnDMode_T *aDnDMode);
     HRESULT setDnDMode(DnDMode_T aDnDMode);
     HRESULT getTeleporterEnabled(BOOL *aTeleporterEnabled);
@@ -953,16 +935,6 @@ private:
     HRESULT setParavirtProvider(ParavirtProvider_T aParavirtProvider);
     HRESULT getParavirtDebug(com::Utf8Str &aParavirtDebug);
     HRESULT setParavirtDebug(const com::Utf8Str &aParavirtDebug);
-    HRESULT getFaultToleranceState(FaultToleranceState_T *aFaultToleranceState);
-    HRESULT setFaultToleranceState(FaultToleranceState_T aFaultToleranceState);
-    HRESULT getFaultTolerancePort(ULONG *aFaultTolerancePort);
-    HRESULT setFaultTolerancePort(ULONG aFaultTolerancePort);
-    HRESULT getFaultToleranceAddress(com::Utf8Str &aFaultToleranceAddress);
-    HRESULT setFaultToleranceAddress(const com::Utf8Str &aFaultToleranceAddress);
-    HRESULT getFaultTolerancePassword(com::Utf8Str &aFaultTolerancePassword);
-    HRESULT setFaultTolerancePassword(const com::Utf8Str &aFaultTolerancePassword);
-    HRESULT getFaultToleranceSyncInterval(ULONG *aFaultToleranceSyncInterval);
-    HRESULT setFaultToleranceSyncInterval(ULONG aFaultToleranceSyncInterval);
     HRESULT getRTCUseUTC(BOOL *aRTCUseUTC);
     HRESULT setRTCUseUTC(BOOL aRTCUseUTC);
     HRESULT getIOCacheEnabled(BOOL *aIOCacheEnabled);
@@ -986,15 +958,15 @@ private:
     HRESULT getDefaultFrontend(com::Utf8Str &aDefaultFrontend);
     HRESULT setDefaultFrontend(const com::Utf8Str &aDefaultFrontend);
     HRESULT getUSBProxyAvailable(BOOL *aUSBProxyAvailable);
-    HRESULT getVMProcessPriority(com::Utf8Str &aVMProcessPriority);
-    HRESULT setVMProcessPriority(const com::Utf8Str &aVMProcessPriority);
+    HRESULT getVMProcessPriority(VMProcPriority_T *aVMProcessPriority);
+    HRESULT setVMProcessPriority(VMProcPriority_T aVMProcessPriority);
 
     // wrapped IMachine methods
     HRESULT lockMachine(const ComPtr<ISession> &aSession,
                         LockType_T aLockType);
     HRESULT launchVMProcess(const ComPtr<ISession> &aSession,
                             const com::Utf8Str &aType,
-                            const com::Utf8Str &aEnvironment,
+                            const std::vector<com::Utf8Str> &aEnvironmentChanges,
                             ComPtr<IProgress> &aProgress);
     HRESULT setBootOrder(ULONG aPosition,
                          DeviceType_T aDevice);
@@ -1136,7 +1108,8 @@ private:
     HRESULT createSharedFolder(const com::Utf8Str &aName,
                                const com::Utf8Str &aHostPath,
                                BOOL aWritable,
-                               BOOL aAutomount);
+                               BOOL aAutomount,
+                               const com::Utf8Str &aAutoMountPoint);
     HRESULT removeSharedFolder(const com::Utf8Str &aName);
     HRESULT canShowConsoleWindow(BOOL *aCanShow);
     HRESULT showConsoleWindow(LONG64 *aWinId);
@@ -1195,6 +1168,9 @@ private:
                     CloneMode_T aMode,
                     const std::vector<CloneOptions_T> &aOptions,
                     ComPtr<IProgress> &aProgress);
+    HRESULT moveTo(const com::Utf8Str &aTargetPath,
+                   const com::Utf8Str &aType,
+                   ComPtr<IProgress> &aProgress);
     HRESULT saveState(ComPtr<IProgress> &aProgress);
     HRESULT adoptSavedState(const com::Utf8Str &aSavedStateFile);
     HRESULT discardSavedState(BOOL aFRemoveFile);
@@ -1235,6 +1211,12 @@ private:
     HRESULT onSessionEnd(const ComPtr<ISession> &aSession,
                          ComPtr<IProgress> &aProgress);
     HRESULT finishOnlineMergeMedium();
+    HRESULT clipboardAreaRegister(const std::vector<com::Utf8Str> &aParms, ULONG *aID);
+    HRESULT clipboardAreaUnregister(ULONG aID);
+    HRESULT clipboardAreaAttach(ULONG aID);
+    HRESULT clipboardAreaDetach(ULONG aID);
+    HRESULT clipboardAreaGetMostRecent(ULONG *aID);
+    HRESULT clipboardAreaGetRefCount(ULONG aID, ULONG *aRefCount);
     HRESULT pullGuestProperties(std::vector<com::Utf8Str> &aNames,
                                 std::vector<com::Utf8Str> &aValues,
                                 std::vector<LONG64> &aTimestamps,
@@ -1337,14 +1319,15 @@ public:
     HRESULT i_onNATRedirectRuleChange(ULONG ulSlot, BOOL aNatRuleRemove, IN_BSTR aRuleName,
                                       NATProtocol_T aProto, IN_BSTR aHostIp, LONG aHostPort,
                                       IN_BSTR aGuestIp, LONG aGuestPort);
-    HRESULT i_onStorageControllerChange();
+    HRESULT i_onStorageControllerChange(const com::Guid &aMachineId, const com::Utf8Str &aControllerName);
     HRESULT i_onMediumChange(IMediumAttachment *aMediumAttachment, BOOL aForce);
+    HRESULT i_onVMProcessPriorityChange(VMProcPriority_T aPriority);
     HRESULT i_onAudioAdapterChange(IAudioAdapter *audioAdapter);
     HRESULT i_onSerialPortChange(ISerialPort *serialPort);
     HRESULT i_onParallelPortChange(IParallelPort *parallelPort);
     HRESULT i_onCPUChange(ULONG aCPU, BOOL aRemove);
     HRESULT i_onVRDEServerChange(BOOL aRestart);
-    HRESULT i_onVideoCaptureChange();
+    HRESULT i_onRecordingChange(BOOL aEnable);
     HRESULT i_onUSBControllerChange();
     HRESULT i_onUSBDeviceAttach(IUSBDevice *aDevice,
                                 IVirtualBoxErrorInfo *aError,
@@ -1354,6 +1337,7 @@ public:
                                 IVirtualBoxErrorInfo *aError);
     HRESULT i_onSharedFolderChange();
     HRESULT i_onClipboardModeChange(ClipboardMode_T aClipboardMode);
+    HRESULT i_onClipboardFileTransferModeChange(BOOL aEnable);
     HRESULT i_onDnDModeChange(DnDMode_T aDnDMode);
     HRESULT i_onBandwidthGroupChange(IBandwidthGroup *aBandwidthGroup);
     HRESULT i_onStorageDeviceChange(IMediumAttachment *aMediumAttachment, BOOL aRemove, BOOL aSilent);
@@ -1389,6 +1373,12 @@ private:
     HRESULT onSessionEnd(const ComPtr<ISession> &aSession,
                          ComPtr<IProgress> &aProgress);
     HRESULT finishOnlineMergeMedium();
+    HRESULT clipboardAreaRegister(const std::vector<com::Utf8Str> &aParms, ULONG *aID);
+    HRESULT clipboardAreaUnregister(ULONG aID);
+    HRESULT clipboardAreaAttach(ULONG aID);
+    HRESULT clipboardAreaDetach(ULONG aID);
+    HRESULT clipboardAreaGetMostRecent(ULONG *aID);
+    HRESULT clipboardAreaGetRefCount(ULONG aID, ULONG *aRefCount);
     HRESULT pullGuestProperties(std::vector<com::Utf8Str> &aNames,
                                 std::vector<com::Utf8Str> &aValues,
                                 std::vector<LONG64> &aTimestamps,
@@ -1604,5 +1594,5 @@ inline const Guid &Machine::i_getSnapshotId() const
 }
 
 
-#endif // ____H_MACHINEIMPL
+#endif /* !MAIN_INCLUDED_MachineImpl_h */
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
