@@ -84,6 +84,9 @@ static struct
     { ovf::CIMOSType_CIMOS_OS2,                                  VBOXOSTYPE_OS2Warp3 },
     { ovf::CIMOSType_CIMOS_OS2,                                  VBOXOSTYPE_OS2Warp4 },
     { ovf::CIMOSType_CIMOS_OS2,                                  VBOXOSTYPE_OS2Warp45 },
+    { ovf::CIMOSType_CIMOS_OS2,                                  VBOXOSTYPE_OS21x },
+    { ovf::CIMOSType_CIMOS_OS2,                                  VBOXOSTYPE_ECS },
+    { ovf::CIMOSType_CIMOS_OS2,                                  VBOXOSTYPE_ArcaOS },
     { ovf::CIMOSType_CIMOS_MSDOS,                                VBOXOSTYPE_DOS },
     { ovf::CIMOSType_CIMOS_WIN3x,                                VBOXOSTYPE_Win31 },
     { ovf::CIMOSType_CIMOS_WIN95,                                VBOXOSTYPE_Win95 },
@@ -1823,3 +1826,22 @@ const settings::MachineConfigFile* VirtualSystemDescription::i_getMachineConfig(
     return m->pConfig;
 }
 
+/**
+ * Private method; walks through the array of VirtualSystemDescriptionEntry entries
+ * and returns the one matching the given index.
+ */
+const VirtualSystemDescriptionEntry* VirtualSystemDescription::i_findByIndex(const uint32_t aIndex)
+{
+    vector<VirtualSystemDescriptionEntry>::const_iterator it;
+    for (it = m->maDescriptions.begin();
+         it != m->maDescriptions.end();
+         ++it)
+    {
+        const VirtualSystemDescriptionEntry &d = *it;
+        if (d.ulIndex == aIndex)
+            return &d;
+    }
+
+    return NULL;
+}
+
