@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2018 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -50,7 +50,7 @@
 # define RTTIMER_LINUX_WITH_HRTIMER
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+#if RTLNX_VER_MAX(2,6,31)
 # define mod_timer_pinned               mod_timer
 # define HRTIMER_MODE_ABS_PINNED        HRTIMER_MODE_ABS
 #endif
@@ -116,6 +116,8 @@ typedef struct RTTIMERLNXSUBTIMER
             /** The start of the current run (ns).
              * This is used to calculate when the timer ought to fire the next time. */
             uint64_t                u64NextTS;
+            /** When the timer was started. */
+            uint64_t                nsStartTS;
             /** The u64NextTS in jiffies. */
             unsigned long           ulNextJiffies;
             /** Set when starting or changing the timer so that u64StartTs
@@ -344,7 +346,10 @@ static void rtTimerLnxStartSubTimer(PRTTIMERLNXSUBTIMER pSubTimer, uint64_t u64N
      */
     uint64_t u64NextTS = u64Now + u64First;
     if (!fHighRes)
+    {
         pSubTimer->u.Std.u64NextTS = u64NextTS;
+        pSubTimer->u.Std.nsStartTS = u64NextTS;
+    }
     RTTIMERLNX_LOG(("startsubtimer %p\n", pSubTimer->pParent));
 
     pSubTimer->iTick = 0;
@@ -362,7 +367,7 @@ static void rtTimerLnxStartSubTimer(PRTTIMERLNXSUBTIMER pSubTimer, uint64_t u64N
 #ifdef CONFIG_SMP
         if (fPinned)
         {
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0)
+# if RTLNX_VER_MIN(4,8,0)
             mod_timer(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
 # else
             mod_timer_pinned(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
@@ -544,7 +549,7 @@ static void rtTimerLnxCallbackHandleMigration(PRTTIMER pTimer, PRTTIMERLNXSUBTIM
                 break;
 
             default:
-                AssertMsgFailed(("%d\n", enmState));
+                AssertMsgFailed(("%d\n", enmState)); RT_FALL_THRU();
             case RTTIMERLNXSTATE_STARTING:
             case RTTIMERLNXSTATE_MP_STARTING:
             case RTTIMERLNXSTATE_ACTIVE:
@@ -596,7 +601,7 @@ static bool rtTimerLnxChangeToCallbackStateSlow(PRTTIMERLNXSUBTIMER pSubTimer)
             case RTTIMERLNXSTATE_CB_STOPPING:
             case RTTIMERLNXSTATE_CB_RESTARTING:
             case RTTIMERLNXSTATE_CB_DESTROYING:
-                AssertMsgFailed(("%d\n", enmState));
+                AssertMsgFailed(("%d\n", enmState)); RT_FALL_THRU();
             default:
                 return false;
         }
@@ -715,7 +720,7 @@ static enum hrtimer_restart rtTimerLinuxHrCallback(struct hrtimer *pHrTimer)
 #endif /* RTTIMER_LINUX_WITH_HRTIMER */
 
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0)
+#if RTLNX_VER_MIN(4,15,0)
 /**
  * Timer callback function for standard timers.
  *
@@ -765,7 +770,9 @@ static void rtTimerLinuxStdCallback(unsigned long ulUser)
          * The first time around, we'll re-adjust the u.Std.u64NextTS to
          * try prevent some jittering if we were started at a bad time.
          */
-        const uint64_t  iTick = ++pSubTimer->iTick;
+        const uint64_t  iTick       = ++pSubTimer->iTick;
+        unsigned long   uCurJiffies = jiffies;
+        unsigned long   ulNextJiffies;
         uint64_t        u64NanoInterval;
         unsigned long   cJiffies;
         unsigned long   flFlags;
@@ -777,28 +784,46 @@ static void rtTimerLinuxStdCallback(unsigned long ulUser)
         {
             pSubTimer->u.Std.fFirstAfterChg = false;
             pSubTimer->u.Std.u64NextTS      = RTTimeSystemNanoTS();
-            pSubTimer->u.Std.ulNextJiffies  = jiffies;
+            pSubTimer->u.Std.nsStartTS      = pSubTimer->u.Std.u64NextTS - u64NanoInterval * (iTick - 1);
+            pSubTimer->u.Std.ulNextJiffies  = uCurJiffies = jiffies;
         }
         spin_unlock_irqrestore(&pTimer->ChgIntLock, flFlags);
 
         pSubTimer->u.Std.u64NextTS += u64NanoInterval;
         if (cJiffies)
         {
-            pSubTimer->u.Std.ulNextJiffies += cJiffies;
-            /* Prevent overflows when the jiffies counter wraps around.
-             * Special thanks to Ken Preslan for helping debugging! */
-            while (time_before(pSubTimer->u.Std.ulNextJiffies, jiffies))
+            ulNextJiffies = pSubTimer->u.Std.ulNextJiffies + cJiffies;
+            pSubTimer->u.Std.ulNextJiffies = ulNextJiffies;
+            if (time_after_eq(ulNextJiffies, uCurJiffies))
+            { /* likely */ }
+            else
             {
-                pSubTimer->u.Std.ulNextJiffies += cJiffies;
-                pSubTimer->u.Std.u64NextTS     += u64NanoInterval;
+                unsigned long cJiffiesBehind = uCurJiffies - ulNextJiffies;
+                ulNextJiffies = uCurJiffies + cJiffies / 2;
+                if (cJiffiesBehind >= HZ / 4) /* Conside if we're lagging too far behind.  Screw the u64NextTS member. */
+                    pSubTimer->u.Std.ulNextJiffies = ulNextJiffies;
+                /*else: Don't update u.Std.ulNextJiffies so we can continue catching up in the next tick. */
             }
         }
         else
         {
             const uint64_t u64NanoTS = RTTimeSystemNanoTS();
-            while (pSubTimer->u.Std.u64NextTS < u64NanoTS)
-                pSubTimer->u.Std.u64NextTS += u64NanoInterval;
-            pSubTimer->u.Std.ulNextJiffies = jiffies + rtTimerLnxNanoToJiffies(pSubTimer->u.Std.u64NextTS - u64NanoTS);
+            const int64_t  cNsBehind = u64NanoTS - pSubTimer->u.Std.u64NextTS;
+            if (cNsBehind <= 0)
+                ulNextJiffies = uCurJiffies + rtTimerLnxNanoToJiffies(pSubTimer->u.Std.u64NextTS - u64NanoTS);
+            else if (u64NanoInterval >= RT_NS_1SEC_64 * 2 / HZ)
+            {
+                ulNextJiffies = uCurJiffies + rtTimerLnxNanoToJiffies(u64NanoInterval / 2);
+                if (cNsBehind >= RT_NS_1SEC_64 / HZ / 4) /* Conside if we're lagging too far behind. */
+                    pSubTimer->u.Std.u64NextTS = u64NanoTS + u64NanoInterval / 2;
+            }
+            else
+            {
+                ulNextJiffies = uCurJiffies + 1;
+                if (cNsBehind >= RT_NS_1SEC_64 / HZ / 4) /* Conside if we're lagging too far behind. */
+                    pSubTimer->u.Std.u64NextTS = u64NanoTS + RT_NS_1SEC_64 / HZ;
+            }
+            pSubTimer->u.Std.ulNextJiffies = ulNextJiffies;
         }
 
         /*
@@ -814,15 +839,15 @@ static void rtTimerLinuxStdCallback(unsigned long ulUser)
 #ifdef CONFIG_SMP
             if (pTimer->fSpecificCpu || pTimer->fAllCpus)
             {
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0)
-                mod_timer(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
+# if RTLNX_VER_MIN(4,8,0)
+                mod_timer(&pSubTimer->u.Std.LnxTimer, ulNextJiffies);
 # else
-                mod_timer_pinned(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
+                mod_timer_pinned(&pSubTimer->u.Std.LnxTimer, ulNextJiffies);
 # endif
             }
             else
 #endif
-                mod_timer(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
+                mod_timer(&pSubTimer->u.Std.LnxTimer, ulNextJiffies);
             return;
         }
     }
@@ -877,7 +902,7 @@ static void rtTimerLinuxStdCallback(unsigned long ulUser)
 #ifdef CONFIG_SMP
                     if (pTimer->fSpecificCpu || pTimer->fAllCpus)
                     {
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0)
+# if RTLNX_VER_MIN(4,8,0)
                         mod_timer(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
 # else
                         mod_timer_pinned(&pSubTimer->u.Std.LnxTimer, pSubTimer->u.Std.ulNextJiffies);
@@ -1418,8 +1443,8 @@ RTDECL(int) RTTimerChangeInterval(PRTTIMER pTimer, uint64_t u64NanoInterval)
     if (pTimer->cCpus > 1)
         return VERR_NOT_SUPPORTED;
 
-    cJiffies = u64NanoInterval / RTTimerGetSystemGranularity();
-    if (cJiffies * RTTimerGetSystemGranularity() != u64NanoInterval)
+    cJiffies = u64NanoInterval / (RT_NS_1SEC / HZ);
+    if (cJiffies * (RT_NS_1SEC / HZ) != u64NanoInterval)
         cJiffies = 0;
 
     spin_lock_irqsave(&pTimer->ChgIntLock, flFlags);
@@ -1494,7 +1519,7 @@ RTDECL(int) RTTimerDestroy(PRTTIMER pTimer)
     {
         /* For paranoid reasons, defer actually destroying the semaphore when
            in atomic or interrupt context. */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 32)
+#if RTLNX_VER_MIN(2,5,32)
         if (in_atomic() || in_interrupt())
 #else
         if (in_interrupt())
@@ -1579,8 +1604,8 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
     pTimer->pfnTimer        = pfnTimer;
     pTimer->pvUser          = pvUser;
     pTimer->u64NanoInterval = u64NanoInterval;
-    pTimer->cJiffies        = u64NanoInterval / RTTimerGetSystemGranularity();
-    if (pTimer->cJiffies * RTTimerGetSystemGranularity() != u64NanoInterval)
+    pTimer->cJiffies        = u64NanoInterval / (RT_NS_1SEC / HZ);
+    if (pTimer->cJiffies * (RT_NS_1SEC / HZ) != u64NanoInterval)
         pTimer->cJiffies    = 0;
     spin_lock_init(&pTimer->ChgIntLock);
 
@@ -1595,14 +1620,14 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
         else
 #endif
         {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0)
+#if RTLNX_VER_MIN(4,15,0)
             timer_setup(&pTimer->aSubTimers[iCpu].u.Std.LnxTimer, rtTimerLinuxStdCallback, TIMER_PINNED);
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0)
+#elif RTLNX_VER_MIN(4,8,0)
             init_timer_pinned(&pTimer->aSubTimers[iCpu].u.Std.LnxTimer);
 #else
             init_timer(&pTimer->aSubTimers[iCpu].u.Std.LnxTimer);
 #endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 15, 0)
+#if RTLNX_VER_MAX(4,15,0)
             pTimer->aSubTimers[iCpu].u.Std.LnxTimer.data        = (unsigned long)&pTimer->aSubTimers[iCpu];
             pTimer->aSubTimers[iCpu].u.Std.LnxTimer.function    = rtTimerLinuxStdCallback;
 #endif
@@ -1660,7 +1685,16 @@ RTDECL(uint32_t) RTTimerGetSystemGranularity(void)
         return Ts.tv_nsec;
     }
 #endif
+    /* */
+#if RTLNX_VER_MAX(4,9,0) || RTLNX_VER_MIN(4,13,0)
+    /* On 4.9, 4.10 and 4.12 we've observed tstRTR0Timer failures of the omni timer tests
+       where we get about half of the ticks we want.  The failing test is using this value
+       as interval.  So, this is a very very crude hack to try make omni timers work
+       correctly without actually knowing what's going wrong... */
+    return RT_NS_1SEC * 2 / HZ; /* ns */
+#else
     return RT_NS_1SEC / HZ; /* ns */
+#endif
 }
 RT_EXPORT_SYMBOL(RTTimerGetSystemGranularity);
 
