@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2017 Oracle Corporation
+ * Copyright (C) 2017-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -236,6 +236,7 @@ static const CUEKEYWORDDESC g_aCueKeywords[] =
 {
     {RT_STR_TUPLE("FILE"),       CUEKEYWORD_FILE},
     {RT_STR_TUPLE("BINARY"),     CUEKEYWORD_BINARY},
+    {RT_STR_TUPLE("MOTOROLA"),   CUEKEYWORD_MOTOROLA},
     {RT_STR_TUPLE("WAVE"),       CUEKEYWORD_WAVE},
     {RT_STR_TUPLE("MP3"),        CUEKEYWORD_MP3},
     {RT_STR_TUPLE("AIFF"),       CUEKEYWORD_AIFF},
@@ -261,6 +262,20 @@ static const CUEKEYWORDDESC g_aCueKeywords[] =
 *   Internal Functions                                                                                                           *
 *********************************************************************************************************************************/
 
+
+/**
+ * Converts a MSF formatted address value read from the given buffer
+ * to an LBA number. MSF 00:00:00 equals LBA 0.
+ *
+ * @returns The LBA number.
+ * @param   pbBuf               The buffer to read the MSF formatted address
+ *                              from.
+ */
+DECLINLINE(uint32_t) cueMSF2LBA(const uint8_t *pbBuf)
+{
+    return (pbBuf[0] * 60 + pbBuf[1]) * 75 + pbBuf[2];
+}
+
 /**
  * Ensures that the region list can hold up to the given number of tracks.
  *
@@ -836,7 +851,7 @@ static int cueParseIndex(PCUEIMAGE pThis, PCUETOKENIZER pTokenizer,
                 abMsf[2] = pToken->Type.Msf.u8Frame;
 
                 *pu8Index = (uint8_t)u64Index;
-                *pu64Lba  = scsiMSF2LBA(&abMsf[0]);
+                *pu64Lba  = cueMSF2LBA(&abMsf[0]);
                 cueTokenizerConsume(pTokenizer);
             }
             else
@@ -1017,6 +1032,16 @@ static int cueParseTrackList(PCUEIMAGE pThis, PCUETOKENIZER pTokenizer)
 {
     int rc = VINF_SUCCESS;
 
+    /*
+     * Sometimes there is a TITLE/PERFORMER/SONGWRITER directive before the start of the track list,
+     * skip and ignore those.
+     */
+    while (   RT_SUCCESS(rc)
+           && (   cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_TITLE)
+               || cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_PERFORMER)
+               || cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_SONGWRITER)))
+        rc = cueParseAndSkipStringRemainder(pThis, pTokenizer, "TITLE/PERFORMER/SONGWRITER");
+
     while (   RT_SUCCESS(rc)
            && cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_TRACK))
         rc = cueParseTrack(pThis, pTokenizer);
@@ -1085,12 +1110,6 @@ static int cueParseKeyword(PCUEIMAGE pThis, PCUETOKENIZER pTokenizer)
 
     if (cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_FILE))
         rc = cueParseFile(pThis, pTokenizer);
-    else if (cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_TITLE))
-        rc = cueParseAndSkipStringRemainder(pThis, pTokenizer, "TITLE");
-    else if (cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_PERFORMER))
-        rc = cueParseAndSkipStringRemainder(pThis, pTokenizer, "PERFORMER");
-    else if (cueTokenizerSkipIfIsKeywordEqual(pTokenizer, CUEKEYWORD_SONGWRITER))
-        rc = cueParseAndSkipStringRemainder(pThis, pTokenizer, "SONGWRITER");
     else /* Skip all other keywords we don't need/support. */
         cueTokenizerConsume(pTokenizer);
 
@@ -1417,9 +1436,9 @@ static PCVDREGIONDESC cueRegionQueryByOffset(PCUEIMAGE pThis, uint64_t uOffset)
 
 /** @copydoc VDIMAGEBACKEND::pfnProbe */
 static DECLCALLBACK(int) cueProbe(const char *pszFilename, PVDINTERFACE pVDIfsDisk,
-                                  PVDINTERFACE pVDIfsImage, VDTYPE *penmType)
+                                  PVDINTERFACE pVDIfsImage, VDTYPE enmDesiredType, VDTYPE *penmType)
 {
-    RT_NOREF1(pVDIfsDisk);
+    RT_NOREF(pVDIfsDisk, enmDesiredType);
     LogFlowFunc(("pszFilename=\"%s\" pVDIfsDisk=%#p pVDIfsImage=%#p\n", pszFilename, pVDIfsDisk, pVDIfsImage));
     int rc = VINF_SUCCESS;
 
@@ -1516,9 +1535,9 @@ static DECLCALLBACK(int) cueRead(void *pBackendData, uint64_t uOffset, size_t cb
         cbToRead = RT_MIN(cbToRead, pRegion->cRegionBlocksOrBytes - offRead);
         Assert(!(cbToRead % pRegion->cbBlock));
 
-        /* Need to convert audio data samples to big endian. */
+        /* Need to convert audio data samples to little endian. */
         if (   pRegion->enmDataForm == VDREGIONDATAFORM_CDDA
-            && pThis->fLittleEndian)
+            && !pThis->fLittleEndian)
         {
             *pcbActuallyRead = cbToRead;
 
