@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 # $Id: utils.py $
-# pylint: disable=C0302
+# pylint: disable=too-many-lines
 
 """
 Common Utility Functions.
 """
 
+from __future__ import print_function;
+
 __copyright__ = \
 """
-Copyright (C) 2012-2017 Oracle Corporation
+Copyright (C) 2012-2020 Oracle Corporation
 
 This file is part of VirtualBox Open Source Edition (OSE), as
 available from http://www.virtualbox.org. This file is free software;
@@ -27,7 +29,7 @@ CDDL are applicable instead of those of the GPL.
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
 """
-__version__ = "$Revision: 118928 $"
+__version__ = "$Revision: 135976 $"
 
 
 # Standard Python imports.
@@ -62,6 +64,53 @@ if sys.version_info[0] >= 3:
     long = int;     # pylint: disable=redefined-builtin,invalid-name
 
 
+#
+# Strings.
+#
+
+def toUnicode(sString, encoding = None, errors = 'strict'):
+    """
+    A little like the python 2 unicode() function.
+    """
+    if sys.version_info[0] >= 3:
+        if isinstance(sString, bytes):
+            return str(sString, encoding if encoding else 'utf-8', errors);
+    else:
+        if not isinstance(sString, unicode):
+            return unicode(sString, encoding if encoding else 'utf-8', errors);
+    return sString;
+
+
+
+#
+# Output.
+#
+
+def printOut(sString):
+    """
+    Outputs a string to standard output, dealing with python 2.x encoding stupidity.
+    """
+    sStreamEncoding = sys.stdout.encoding;
+    if sStreamEncoding is None:         # Files, pipes and such on 2.x.  (pylint is confused here)
+        sStreamEncoding = 'US-ASCII';   # pylint: disable=redefined-variable-type
+    if sStreamEncoding == 'UTF-8' or not isinstance(sString, unicode):
+        print(sString);
+    else:
+        print(sString.encode(sStreamEncoding, 'backslashreplace').decode(sStreamEncoding));
+
+def printErr(sString):
+    """
+    Outputs a string to standard error, dealing with python 2.x encoding stupidity.
+    """
+    sStreamEncoding = sys.stderr.encoding;
+    if sStreamEncoding is None:         # Files, pipes and such on 2.x. (pylint is confused here)
+        sStreamEncoding = 'US-ASCII';   # pylint: disable=redefined-variable-type
+    if sStreamEncoding == 'UTF-8' or not isinstance(sString, unicode):
+        print(sString, file = sys.stderr);
+    else:
+        print(sString.encode(sStreamEncoding, 'backslashreplace').decode(sStreamEncoding), file = sys.stderr);
+
+
 #
 # Host OS and CPU.
 #
@@ -103,7 +152,7 @@ def getHostArch():
                 sArch = 'amd64';
             else:
                 try:
-                    sArch = processOutputChecked(['/usr/bin/isainfo', '-n',]);
+                    sArch = str(processOutputChecked(['/usr/bin/isainfo', '-n',]));
                 except:
                     pass;
                 sArch = sArch.strip();
@@ -268,6 +317,29 @@ def getHostOsVersion():
 
     return sVersion;
 
+def getPresentCpuCount():
+    """
+    Gets the number of CPUs present in the system.
+
+    This differs from multiprocessor.cpu_count() and os.cpu_count() on windows in
+    that we return the active count rather than the maximum count.  If we don't,
+    we will end up thinking testboxmem1 has 512 CPU threads, which it doesn't and
+    never will have.
+
+    @todo This is probably not exactly what we get on non-windows...
+    """
+
+    if getHostOs() == 'win':
+        fnGetActiveProcessorCount = getattr(ctypes.windll.kernel32, 'GetActiveProcessorCount', None);
+        if fnGetActiveProcessorCount:
+            cCpus = fnGetActiveProcessorCount(ctypes.c_ushort(0xffff));
+            if cCpus > 0:
+                return cCpus;
+
+    import multiprocessing
+    return multiprocessing.cpu_count();
+
+
 #
 # File system.
 #
@@ -282,24 +354,30 @@ def openNoInherit(sFile, sMode = 'r'):
     multithreaded programs.
     """
 
-    try:
-        from fcntl import FD_CLOEXEC, F_GETFD, F_SETFD, fcntl; # pylint: disable=F0401
-    except:
-        # On windows, use the 'N' flag introduces in Visual C++ 7.0 or 7.1.
-        if getHostOs() == 'win':
-            offComma = sMode.find(',');
-            if offComma < 0:
-                return open(sFile, sMode + 'N');
-            return open(sFile, sMode[:offComma] + 'N' + sMode[offComma:]);
+    # Python 3.4 and later automatically creates non-inherit handles. See PEP-0446.
+    uPythonVer = (sys.version_info[0] << 16) | (sys.version_info[1] & 0xffff);
+    if uPythonVer >= ((3 << 16) | 4):
+        oFile = open(sFile, sMode);
+    else:
+        try:
+            from fcntl import FD_CLOEXEC, F_GETFD, F_SETFD, fcntl; # pylint: disable=import-error
+        except:
+            # On windows, we can use the 'N' flag introduced in Visual C++ 7.0 or 7.1 with python 2.x.
+            if getHostOs() == 'win':
+                if uPythonVer < (3 << 16):
+                    offComma = sMode.find(',');
+                    if offComma < 0:
+                        return open(sFile, sMode + 'N');
+                    return open(sFile, sMode[:offComma] + 'N' + sMode[offComma:]); # pylint: disable=bad-open-mode
 
-        # Just in case.
-        return open(sFile, sMode);
+            # Just in case.
+            return open(sFile, sMode);
 
-    oFile = open(sFile, sMode)
-    #try:
-    fcntl(oFile, F_SETFD, fcntl(oFile, F_GETFD) | FD_CLOEXEC);
-    #except:
-    #    pass;
+        oFile = open(sFile, sMode);
+        #try:
+        fcntl(oFile, F_SETFD, fcntl(oFile, F_GETFD) | FD_CLOEXEC);
+        #except:
+        #    pass;
     return oFile;
 
 def openNoDenyDeleteNoInherit(sFile, sMode = 'r'):
@@ -312,68 +390,72 @@ def openNoDenyDeleteNoInherit(sFile, sMode = 'r'):
     multithreaded programs.
     """
 
-    try:
-        from fcntl import FD_CLOEXEC, F_GETFD, F_SETFD, fcntl; # pylint: disable=F0401
-    except:
-        if getHostOs() == 'win':
-            # Need to use CreateFile directly to open the file so we can feed it FILE_SHARE_DELETE.
-            fAccess = 0;
-            fDisposition = win32file.OPEN_EXISTING;                                                 # pylint: disable=no-member
-            if 'r' in sMode or '+' in sMode:
-                fAccess |= win32file.GENERIC_READ;                                                  # pylint: disable=no-member
-            if 'a' in sMode:
-                fAccess |= win32file.GENERIC_WRITE;                                                 # pylint: disable=no-member
-                fDisposition = win32file.OPEN_ALWAYS;                                               # pylint: disable=no-member
-            elif 'w' in sMode:
-                fAccess = win32file.GENERIC_WRITE;                                                  # pylint: disable=no-member
-                if '+' in sMode:
-                    fDisposition = win32file.OPEN_ALWAYS;                                           # pylint: disable=no-member
-                    fAccess |= win32file.GENERIC_READ;                                              # pylint: disable=no-member
-                else:
-                    fDisposition = win32file.CREATE_ALWAYS;                                         # pylint: disable=no-member
-            if not fAccess:
-                fAccess |= win32file.GENERIC_READ;                                                  # pylint: disable=no-member
-            fSharing = (win32file.FILE_SHARE_READ | win32file.FILE_SHARE_WRITE                      # pylint: disable=no-member
-                        | win32file.FILE_SHARE_DELETE);                                             # pylint: disable=no-member
-            hFile = win32file.CreateFile(sFile, fAccess, fSharing, None, fDisposition, 0, None);    # pylint: disable=no-member
-            if 'a' in sMode:
-                win32file.SetFilePointer(hFile, 0, win32file.FILE_END);                             # pylint: disable=no-member
-
-            # Turn the NT handle into a CRT file descriptor.
-            hDetachedFile = hFile.Detach();
-            if fAccess == win32file.GENERIC_READ:                                                   # pylint: disable=no-member
-                fOpen = os.O_RDONLY;
-            elif fAccess == win32file.GENERIC_WRITE:                                                # pylint: disable=no-member
-                fOpen = os.O_WRONLY;
+    if getHostOs() == 'win':
+        # Need to use CreateFile directly to open the file so we can feed it FILE_SHARE_DELETE.
+        # pylint: disable=no-member,c-extension-no-member
+        fAccess = 0;
+        fDisposition = win32file.OPEN_EXISTING;
+        if 'r' in sMode or '+' in sMode:
+            fAccess |= win32file.GENERIC_READ;
+        if 'a' in sMode:
+            fAccess |= win32file.GENERIC_WRITE;
+            fDisposition = win32file.OPEN_ALWAYS;
+        elif 'w' in sMode:
+            fAccess = win32file.GENERIC_WRITE;
+            if '+' in sMode:
+                fDisposition = win32file.OPEN_ALWAYS;
+                fAccess |= win32file.GENERIC_READ;
             else:
-                fOpen = os.O_RDWR;
-            if 'a' in sMode:
-                fOpen |= os.O_APPEND;
-            if 'b' in sMode or 't' in sMode:
-                fOpen |= os.O_TEXT;                                                                 # pylint: disable=no-member
-            fdFile = msvcrt.open_osfhandle(hDetachedFile, fOpen);
+                fDisposition = win32file.CREATE_ALWAYS;
+        if not fAccess:
+            fAccess |= win32file.GENERIC_READ;
+        fSharing = (win32file.FILE_SHARE_READ | win32file.FILE_SHARE_WRITE
+                    | win32file.FILE_SHARE_DELETE);
+        hFile = win32file.CreateFile(sFile, fAccess, fSharing, None, fDisposition, 0, None);
+        if 'a' in sMode:
+            win32file.SetFilePointer(hFile, 0, win32file.FILE_END);
 
-            # Tell python to use this handle.
-            return os.fdopen(fdFile, sMode);
+        # Turn the NT handle into a CRT file descriptor.
+        hDetachedFile = hFile.Detach();
+        if fAccess == win32file.GENERIC_READ:
+            fOpen = os.O_RDONLY;
+        elif fAccess == win32file.GENERIC_WRITE:
+            fOpen = os.O_WRONLY;
+        else:
+            fOpen = os.O_RDWR;
+        # pulint: enable=no-member,c-extension-no-member
+        if 'a' in sMode:
+            fOpen |= os.O_APPEND;
+        if 'b' in sMode or 't' in sMode:
+            fOpen |= os.O_TEXT;                                                                 # pylint: disable=no-member
+        fdFile = msvcrt.open_osfhandle(hDetachedFile, fOpen);
 
-        # Just in case.
-        return open(sFile, sMode);
+        # Tell python to use this handle.
+        oFile = os.fdopen(fdFile, sMode);
+    else:
+        oFile = open(sFile, sMode);
 
-    oFile = open(sFile, sMode)
-    #try:
-    fcntl(oFile, F_SETFD, fcntl(oFile, F_GETFD) | FD_CLOEXEC);
-    #except:
-    #    pass;
+        # Python 3.4 and later automatically creates non-inherit handles. See PEP-0446.
+        uPythonVer = (sys.version_info[0] << 16) | (sys.version_info[1] & 0xffff);
+        if uPythonVer < ((3 << 16) | 4):
+            try:
+                from fcntl import FD_CLOEXEC, F_GETFD, F_SETFD, fcntl; # pylint: disable=import-error
+            except:
+                pass;
+            else:
+                fcntl(oFile, F_SETFD, fcntl(oFile, F_GETFD) | FD_CLOEXEC);
     return oFile;
 
-def noxcptReadLink(sPath, sXcptRet):
+def noxcptReadLink(sPath, sXcptRet, sEncoding = 'utf-8'):
     """
     No exceptions os.readlink wrapper.
     """
     try:
-        sRet = os.readlink(sPath); # pylint: disable=E1101
+        sRet = os.readlink(sPath); # pylint: disable=no-member
     except:
-        sRet = sXcptRet;
+        return sXcptRet;
+    if hasattr(sRet, 'decode'):
+        sRet = sRet.decode(sEncoding, 'ignore');
     return sRet;
 
 def readFile(sFile, sMode = 'rb'):
@@ -385,7 +467,7 @@ def readFile(sFile, sMode = 'rb'):
     oFile.close();
     return sRet;
 
-def noxcptReadFile(sFile, sXcptRet, sMode = 'rb'):
+def noxcptReadFile(sFile, sXcptRet, sMode = 'rb', sEncoding = 'utf-8'):
     """
     No exceptions common.readFile wrapper.
     """
@@ -393,6 +475,8 @@ def noxcptReadFile(sFile, sXcptRet, sMode = 'rb'):
         sRet = readFile(sFile, sMode);
     except:
         sRet = sXcptRet;
+    if sEncoding is not None and hasattr(sRet, 'decode'):
+        sRet = sRet.decode(sEncoding, 'ignore');
     return sRet;
 
 def noxcptRmDir(sDir, oXcptRet = False):
@@ -526,6 +610,29 @@ def copyFileSimple(sFileSrc, sFileDst):
     __installShUtilHacks(shutil);
     return shutil.copyfile(sFileSrc, sFileDst);
 
+
+def getDiskUsage(sPath):
+    """
+    Get free space of a partition that corresponds to specified sPath in MB.
+
+    Returns partition free space value in MB.
+    """
+    if platform.system() == 'Windows':
+        oCTypeFreeSpace = ctypes.c_ulonglong(0);
+        ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(sPath), None, None,
+                                                   ctypes.pointer(oCTypeFreeSpace));
+        cbFreeSpace = oCTypeFreeSpace.value;
+    else:
+        oStats = os.statvfs(sPath); # pylint: disable=no-member
+        cbFreeSpace = long(oStats.f_frsize) * oStats.f_bfree;
+
+    # Convert to MB
+    cMbFreeSpace = long(cbFreeSpace) / (1024 * 1024);
+
+    return cMbFreeSpace;
+
+
+
 #
 # SubProcess.
 #
@@ -579,21 +686,35 @@ def processOutputChecked(*aPositionalArgs, **dKeywordArgs):
     """
     Wrapper around subprocess.check_output to deal with its absense in older
     python versions.
+
+    Extra keywords for specifying now output is to be decoded:
+        sEncoding='utf-8
+        fIgnoreEncoding=True/False
     """
+    sEncoding = dKeywordArgs.get('sEncoding');
+    if sEncoding is not None:   del dKeywordArgs['sEncoding'];
+    else:                       sEncoding = 'utf-8';
+
+    fIgnoreEncoding = dKeywordArgs.get('fIgnoreEncoding');
+    if fIgnoreEncoding is not None:   del dKeywordArgs['fIgnoreEncoding'];
+    else:                             fIgnoreEncoding = True;
+
     _processFixPythonInterpreter(aPositionalArgs, dKeywordArgs);
     oProcess = processPopenSafe(stdout=subprocess.PIPE, *aPositionalArgs, **dKeywordArgs);
 
     sOutput, _ = oProcess.communicate();
     iExitCode  = oProcess.poll();
 
-    if iExitCode is not 0:
+    if iExitCode != 0:
         asArgs = dKeywordArgs.get('args');
         if asArgs is None:
             asArgs = aPositionalArgs[0];
         print(sOutput);
         raise subprocess.CalledProcessError(iExitCode, asArgs);
 
-    return str(sOutput); # str() make pylint happy.
+    if hasattr(sOutput, 'decode'):
+        sOutput = sOutput.decode(sEncoding, 'ignore' if fIgnoreEncoding else 'strict');
+    return sOutput;
 
 g_fOldSudo = None;
 def _sudoFixArguments(aPositionalArgs, dKeywordArgs, fInitialEnv = True):
@@ -604,7 +725,7 @@ def _sudoFixArguments(aPositionalArgs, dKeywordArgs, fInitialEnv = True):
     # Are we root?
     fIsRoot = True;
     try:
-        fIsRoot = os.getuid() == 0; # pylint: disable=E1101
+        fIsRoot = os.getuid() == 0; # pylint: disable=no-member
     except:
         pass;
 
@@ -618,7 +739,7 @@ def _sudoFixArguments(aPositionalArgs, dKeywordArgs, fInitialEnv = True):
         global g_fOldSudo;
         if g_fOldSudo is None:
             try:
-                sVersion = processOutputChecked(['sudo', '-V']);
+                sVersion = str(processOutputChecked(['sudo', '-V']));
             except:
                 sVersion = '1.7.0';
             sVersion = sVersion.strip().split('\n')[0];
@@ -690,7 +811,8 @@ def processInterrupt(uPid):
     """
     if sys.platform == 'win32':
         try:
-            win32console.GenerateConsoleCtrlEvent(win32con.CTRL_BREAK_EVENT, uPid);             # pylint: disable=no-member
+            win32console.GenerateConsoleCtrlEvent(win32con.CTRL_BREAK_EVENT, # pylint: disable=no-member,c-extension-no-member
+                                                  uPid);
             fRc = True;
         except:
             fRc = False;
@@ -715,7 +837,7 @@ def sendUserSignal1(uPid):
         fRc = False;
     else:
         try:
-            os.kill(uPid, signal.SIGUSR1); # pylint: disable=E1101
+            os.kill(uPid, signal.SIGUSR1); # pylint: disable=no-member
             fRc = True;
         except:
             fRc = False;
@@ -729,12 +851,14 @@ def processTerminate(uPid):
     fRc = False;
     if sys.platform == 'win32':
         try:
-            hProcess = win32api.OpenProcess(win32con.PROCESS_TERMINATE, False, uPid);           # pylint: disable=no-member
+            hProcess = win32api.OpenProcess(win32con.PROCESS_TERMINATE,     # pylint: disable=no-member,c-extension-no-member
+                                            False, uPid);
         except:
             pass;
         else:
             try:
-                win32process.TerminateProcess(hProcess, 0x40010004); # DBG_TERMINATE_PROCESS    # pylint: disable=no-member
+                win32process.TerminateProcess(hProcess,                     # pylint: disable=no-member,c-extension-no-member
+                                              0x40010004); # DBG_TERMINATE_PROCESS
                 fRc = True;
             except:
                 pass;
@@ -756,7 +880,7 @@ def processKill(uPid):
         fRc = processTerminate(uPid);
     else:
         try:
-            os.kill(uPid, signal.SIGKILL); # pylint: disable=E1101
+            os.kill(uPid, signal.SIGKILL); # pylint: disable=no-member
             fRc = True;
         except:
             fRc = False;
@@ -786,8 +910,9 @@ def processExists(uPid):
         fRc = False;
         # We try open the process for waiting since this is generally only forbidden in a very few cases.
         try:
-            hProcess = win32api.OpenProcess(win32con.SYNCHRONIZE, False, uPid);     # pylint: disable=no-member
-        except pywintypes.error as oXcpt:                                           # pylint: disable=no-member
+            hProcess = win32api.OpenProcess(win32con.SYNCHRONIZE,           # pylint: disable=no-member,c-extension-no-member
+                                            False, uPid);
+        except pywintypes.error as oXcpt:                                   # pylint: disable=no-member
             if oXcpt.winerror == winerror.ERROR_ACCESS_DENIED:
                 fRc = True;
         except Exception as oXcpt:
@@ -813,7 +938,7 @@ def processCheckPidAndName(uPid, sName):
 
     if sys.platform == 'win32':
         try:
-            from win32com.client import GetObject; # pylint: disable=F0401
+            from win32com.client import GetObject; # pylint: disable=import-error
             oWmi = GetObject('winmgmts:');
             aoProcesses = oWmi.InstancesOf('Win32_Process');
             for oProcess in aoProcesses:
@@ -834,7 +959,7 @@ def processCheckPidAndName(uPid, sName):
             #reporter.logXcpt('uPid=%s sName=%s' % (uPid, sName));
             pass;
     else:
-        if sys.platform in ('linux2', ):
+        if sys.platform in ('linux2', 'linux', 'linux3', 'linux4', 'linux5', 'linux6'):
             asPsCmd = ['/bin/ps',     '-p', '%u' % (uPid,), '-o', 'fname='];
         elif sys.platform in ('sunos5',):
             asPsCmd = ['/usr/bin/ps', '-p', '%u' % (uPid,), '-o', 'fname='];
@@ -853,7 +978,7 @@ def processCheckPidAndName(uPid, sName):
                 return False;
 
             # ps fails with non-zero exit code if the pid wasn't found.
-            if iExitCode is not 0:
+            if iExitCode != 0:
                 return False;
             if sCurName is None:
                 return False;
@@ -1007,7 +1132,7 @@ class ProcessInfo(object):
         return sRet;
 
 
-def processListAll(): # pylint: disable=R0914
+def processListAll():
     """
     Return a list of ProcessInfo objects for all the processes in the system
     that the current user can see.
@@ -1016,7 +1141,7 @@ def processListAll(): # pylint: disable=R0914
 
     sOs = getHostOs();
     if sOs == 'win':
-        from win32com.client import GetObject; # pylint: disable=F0401
+        from win32com.client import GetObject; # pylint: disable=import-error
         oWmi = GetObject('winmgmts:');
         aoProcesses = oWmi.InstancesOf('Win32_Process');
         for oProcess in aoProcesses:
@@ -1212,6 +1337,7 @@ def processCollectCrashInfo(uPid, fnLog, fnCrashFile):
 #
 # Note! We cannot use time.clock() as the timestamp must be portable across
 #       processes.  See timeout testcase problem on win hosts (no logs).
+#       Also, time.clock() was axed in python 3.8 (https://bugs.python.org/issue31803).
 #
 #import sys;
 #import time;
@@ -1267,7 +1393,6 @@ def _winFloatTime():
             return float(uCurValue.value) / g_fpWinPerfCounterFreq;
     return time.time();
 
-
 def timestampNano():
     """
     Gets a nanosecond timestamp.
@@ -1292,6 +1417,13 @@ def timestampSecond():
         return long(_winFloatTime());
     return long(time.time());
 
+def secondsSinceUnixEpoch():
+    """
+    Returns unix time, floating point second count since 1970-01-01T00:00:00Z
+    """
+    ## ASSUMES This returns unix epoch time on all systems we care about...
+    return time.time();
+
 def getTimePrefix():
     """
     Returns a timestamp prefix, typically used for logging. UTC.
@@ -1338,18 +1470,18 @@ def formatIntervalSeconds(cSeconds):
     if cSeconds < 60:
         return '%ss' % (cSeconds,);
     if cSeconds < 3600:
-        cMins = cSeconds / 60;
+        cMins = cSeconds // 60;
         cSecs = cSeconds % 60;
         if cSecs == 0:
             return '%sm' % (cMins,);
         return '%sm %ss' % (cMins, cSecs,);
 
     # Generic and a bit slower.
-    cDays     = cSeconds / 86400;
+    cDays     = cSeconds // 86400;
     cSeconds %= 86400;
-    cHours    = cSeconds / 3600;
+    cHours    = cSeconds // 3600;
     cSeconds %= 3600;
-    cMins     = cSeconds / 60;
+    cMins     = cSeconds // 60;
     cSecs     = cSeconds % 60;
     sRet = '';
     if cDays > 0:
@@ -1618,7 +1750,7 @@ def getObjectTypeName(oObject):
     # Python 2.x only: Handle old-style object wrappers.
     if sys.version_info[0] < 3:
         try:
-            from types import InstanceType;
+            from types import InstanceType;     # pylint: disable=no-name-in-module
             if oType == InstanceType:
                 oType = oObject.__class__;
         except:
@@ -1631,6 +1763,24 @@ def getObjectTypeName(oObject):
         return '__type__-throws-exception';
 
 
+def chmodPlusX(sFile):
+    """
+    Makes the specified file or directory executable.
+    Returns success indicator, no exceptions.
+
+    Note! Symbolic links are followed and the target will be changed.
+    """
+    try:
+        oStat = os.stat(sFile);
+    except:
+        return False;
+    try:
+        os.chmod(sFile, oStat.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH);
+    except:
+        return False;
+    return True;
+
+
 #
 # TestSuite stuff.
 #
@@ -1705,10 +1855,6 @@ def stricmp(sFirst, sSecond):
     return 1;
 
 
-#
-# Misc.
-#
-
 def versionCompare(sVer1, sVer2):
     """
     Compares to version strings in a fashion similar to RTStrVersionCompare.
@@ -1753,34 +1899,28 @@ def isString(oString):
     """
     if sys.version_info[0] >= 3:
         return isinstance(oString, str);
-    return isinstance(oString, basestring);
+    return isinstance(oString, basestring);     # pylint: disable=undefined-variable
 
 
 def hasNonAsciiCharacters(sText):
     """
     Returns True is specified string has non-ASCII characters, False if ASCII only.
     """
-    sTmp = unicode(sText, errors='ignore') if isinstance(sText, str) else sText;
-    return not all(ord(ch) < 128 for ch in sTmp);
+    if isString(sText):
+        for ch in sText:
+            if ord(ch) >= 128:
+                return True;
+    else:
+        # Probably byte array or some such thing.
+        for ch in sText:
+            if ch >= 128 or ch < 0:
+                return True;
+    return False;
 
 
-def chmodPlusX(sFile):
-    """
-    Makes the specified file or directory executable.
-    Returns success indicator, no exceptions.
-
-    Note! Symbolic links are followed and the target will be changed.
-    """
-    try:
-        oStat = os.stat(sFile);
-    except:
-        return False;
-    try:
-        os.chmod(sFile, oStat.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH);
-    except:
-        return False;
-    return True;
-
+#
+# Unpacking.
+#
 
 def unpackZipFile(sArchive, sDstDir, fnLog, fnError = None, fnFilter = None):
     # type: (string, string, (string) -> None, (string) -> None, (string) -> bool) -> list[string]
@@ -1825,8 +1965,9 @@ def unpackZipFile(sArchive, sDstDir, fnLog, fnError = None, fnFilter = None):
 ## Set if we've replaced tarfile.copyfileobj with __mytarfilecopyfileobj already.
 g_fTarCopyFileObjOverriddend = False;
 
-def __mytarfilecopyfileobj(src, dst, length = None, exception = OSError):
+def __mytarfilecopyfileobj(src, dst, length = None, exception = OSError, bufsize = None):
     """ tarfile.copyfileobj with different buffer size (16384 is slow on windows). """
+    _ = bufsize;
     if length is None:
         __myshutilcopyfileobj(src, dst, g_cbGoodBufferSize);
     elif length > 0:
@@ -1860,11 +2001,12 @@ def unpackTarFile(sArchive, sDstDir, fnLog, fnError = None, fnFilter = None):
     # 60%+ speedup for python 2.7 and 50%+ speedup for python 3.5, both on windows with PDBs.
     # 20%+ speedup for python 2.7 and 15%+ speedup for python 3.5, both on windows skipping PDBs.
     #
-    if True is True:
+    if True is True: # pylint: disable=comparison-with-itself
         __installShUtilHacks(shutil);
         global g_fTarCopyFileObjOverriddend;
         if g_fTarCopyFileObjOverriddend is False:
             g_fTarCopyFileObjOverriddend = True;
+            #if sys.hexversion < 0x03060000:
             tarfile.copyfileobj = __mytarfilecopyfileobj;
 
     #
@@ -1873,7 +2015,11 @@ def unpackTarFile(sArchive, sDstDir, fnLog, fnError = None, fnFilter = None):
     # Note! We not using 'r:*' because we cannot allow seeking compressed files!
     #       That's how we got a 13 min unpack time for VBoxAll on windows (hardlinked pdb).
     #
-    try: oTarFile = tarfile.open(sArchive, 'r|*', bufsize = g_cbGoodBufferSize);
+    try:
+        if sys.hexversion >= 0x03060000:
+            oTarFile = tarfile.open(sArchive, 'r|*', bufsize = g_cbGoodBufferSize, copybufsize = g_cbGoodBufferSize);
+        else:
+            oTarFile = tarfile.open(sArchive, 'r|*', bufsize = g_cbGoodBufferSize);
     except Exception as oXcpt:
         fnError('Error opening "%s" for unpacking into "%s": %s' % (sArchive, sDstDir, oXcpt,));
         return None;
@@ -1964,32 +2110,113 @@ def unpackFile(sArchive, sDstDir, fnLog, fnError = None, fnFilter = None):
     return [];
 
 
-def getDiskUsage(sPath):
+#
+# Misc.
+#
+def areBytesEqual(oLeft, oRight):
     """
-    Get free space of a partition that corresponds to specified sPath in MB.
+    Compares two byte arrays, strings or whatnot.
 
-    Returns partition free space value in MB.
+    returns true / false accordingly.
     """
-    if platform.system() == 'Windows':
-        oCTypeFreeSpace = ctypes.c_ulonglong(0);
-        ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(sPath), None, None,
-                                                   ctypes.pointer(oCTypeFreeSpace));
-        cbFreeSpace = oCTypeFreeSpace.value;
+
+    # If both are None, consider them equal (bogus?):
+    if oLeft is None and oRight is None:
+        return True;
+
+    # If just one is None, they can't match:
+    if oLeft is None or oRight is None:
+        return False;
+
+    # If both have the same type, use the compare operator of the class:
+    if type(oLeft) is type(oRight):
+        #print('same type: %s' % (oLeft == oRight,));
+        return oLeft == oRight;
+
+    # On the offchance that they're both strings, but of different types.
+    if isString(oLeft) and isString(oRight):
+        #print('string compare: %s' % (oLeft == oRight,));
+        return oLeft == oRight;
+
+    #
+    # See if byte/buffer stuff that can be compared directory. If not convert
+    # strings to bytes.
+    #
+    # Note! For 2.x, we must convert both sides to the buffer type or the
+    #       comparison may fail despite it working okay in test cases.
+    #
+    if sys.version_info[0] >= 3:
+        if isinstance(oLeft, (bytearray, memoryview, bytes)) and isinstance(oRight, (bytearray, memoryview, bytes)):  # pylint: disable=undefined-variable
+            return oLeft == oRight;
+
+        if isString(oLeft):
+            try:    oLeft = bytes(oLeft, 'utf-8');
+            except: pass;
+        if isString(oRight):
+            try:    oRight = bytes(oRight, 'utf-8');
+            except: pass;
     else:
-        oStats = os.statvfs(sPath); # pylint: disable=E1101
-        cbFreeSpace = long(oStats.f_frsize) * oStats.f_bfree;
+        if isinstance(oLeft, (bytearray, buffer)) and isinstance(oRight, (bytearray, buffer)): # pylint: disable=undefined-variable
+            if isinstance(oLeft, bytearray):
+                oLeft  = buffer(oLeft);                     # pylint: disable=redefined-variable-type,undefined-variable
+            else:
+                oRight = buffer(oRight);                    # pylint: disable=redefined-variable-type,undefined-variable
+            #print('buf/byte #1 compare: %s (%s vs %s)' % (oLeft == oRight, type(oLeft), type(oRight),));
+            return oLeft == oRight;
 
-    # Convert to MB
-    cMbFreeSpace = long(cbFreeSpace) / (1024 * 1024);
+        if isString(oLeft):
+            try:    oLeft = bytearray(oLeft, 'utf-8');      # pylint: disable=redefined-variable-type
+            except: pass;
+        if isString(oRight):
+            try:    oRight = bytearray(oRight, 'utf-8');    # pylint: disable=redefined-variable-type
+            except: pass;
 
-    return cMbFreeSpace;
+    # Check if we now have the same type for both:
+    if type(oLeft) is type(oRight):
+        #print('same type now: %s' % (oLeft == oRight,));
+        return oLeft == oRight;
+
+    # Check if we now have buffer/memoryview vs bytes/bytesarray again.
+    if sys.version_info[0] >= 3:
+        if isinstance(oLeft, (bytearray, memoryview, bytes)) and isinstance(oRight, (bytearray, memoryview, bytes)):  # pylint: disable=undefined-variable
+            return oLeft == oRight;
+    else:
+        if isinstance(oLeft, (bytearray, buffer)) and isinstance(oRight, (bytearray, buffer)): # pylint: disable=undefined-variable
+            if isinstance(oLeft, bytearray):
+                oLeft  = buffer(oLeft);                     # pylint: disable=redefined-variable-type,undefined-variable
+            else:
+                oRight = buffer(oRight);                    # pylint: disable=redefined-variable-type,undefined-variable
+            #print('buf/byte #2 compare: %s (%s vs %s)' % (oLeft == oRight, type(oLeft), type(oRight),));
+            return oLeft == oRight;
+
+    # Do item by item comparison:
+    if len(oLeft) != len(oRight):
+        #print('different length: %s vs %s' % (len(oLeft), len(oRight)));
+        return False;
+    i = len(oLeft);
+    while i > 0:
+        i = i - 1;
+
+        iElmLeft = oLeft[i];
+        if not isinstance(iElmLeft, int) and not isinstance(iElmLeft, long):
+            iElmLeft = ord(iElmLeft);
+
+        iElmRight = oRight[i];
+        if not isinstance(iElmRight, int) and not isinstance(iElmRight, long):
+            iElmRight = ord(iElmRight);
+
+        if iElmLeft != iElmRight:
+            #print('element %d differs: %x %x' % (i, iElmLeft, iElmRight,));
+            return False;
+    return True;
 
 
 #
 # Unit testing.
 #
 
-# pylint: disable=C0111
+# pylint: disable=missing-docstring
+# pylint: disable=undefined-variable
 class BuildCategoryDataTestCase(unittest.TestCase):
     def testIntervalSeconds(self):
         self.assertEqual(parseIntervalSeconds(formatIntervalSeconds(3600)), (3600, None));
@@ -2007,10 +2234,45 @@ class BuildCategoryDataTestCase(unittest.TestCase):
     def testHasNonAsciiChars(self):
         self.assertEqual(hasNonAsciiCharacters(''), False);
         self.assertEqual(hasNonAsciiCharacters('asdfgebASDFKJ@#$)(!@#UNASDFKHB*&$%&)@#(!)@(#!(#$&*#$&%*Y@#$IQWN---00;'), False);
+        self.assertEqual(hasNonAsciiCharacters('\x80 '), True);
+        self.assertEqual(hasNonAsciiCharacters('\x79 '), False);
         self.assertEqual(hasNonAsciiCharacters(u'12039889y!@#$%^&*()0-0asjdkfhoiuyweasdfASDFnvV'), False);
         self.assertEqual(hasNonAsciiCharacters(u'\u0079'), False);
         self.assertEqual(hasNonAsciiCharacters(u'\u0080'), True);
         self.assertEqual(hasNonAsciiCharacters(u'\u0081 \u0100'), True);
+        self.assertEqual(hasNonAsciiCharacters(b'\x20\x20\x20'), False);
+        self.assertEqual(hasNonAsciiCharacters(b'\x20\x81\x20'), True);
+
+    def testAreBytesEqual(self):
+        self.assertEqual(areBytesEqual(None, None), True);
+        self.assertEqual(areBytesEqual(None, ''), False);
+        self.assertEqual(areBytesEqual('', ''), True);
+        self.assertEqual(areBytesEqual('1', '1'), True);
+        self.assertEqual(areBytesEqual('12345', '1234'), False);
+        self.assertEqual(areBytesEqual('1234', '1234'), True);
+        self.assertEqual(areBytesEqual('1234', b'1234'), True);
+        self.assertEqual(areBytesEqual(b'1234', b'1234'), True);
+        self.assertEqual(areBytesEqual(b'1234', '1234'), True);
+        self.assertEqual(areBytesEqual(b'1234', bytearray([0x31,0x32,0x33,0x34])), True);
+        self.assertEqual(areBytesEqual('1234', bytearray([0x31,0x32,0x33,0x34])), True);
+        self.assertEqual(areBytesEqual(u'1234', bytearray([0x31,0x32,0x33,0x34])), True);
+        self.assertEqual(areBytesEqual(bytearray([0x31,0x32,0x33,0x34]), bytearray([0x31,0x32,0x33,0x34])), True);
+        self.assertEqual(areBytesEqual(bytearray([0x31,0x32,0x33,0x34]), '1224'), False);
+        self.assertEqual(areBytesEqual(bytearray([0x31,0x32,0x33,0x34]), bytearray([0x31,0x32,0x32,0x34])), False);
+        if sys.version_info[0] >= 3:
+            pass;
+        else:
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1),
+                                                       bytearray([0x31,0x32,0x33,0x34])), True);
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1),
+                                                       bytearray([0x99,0x32,0x32,0x34])), False);
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1),
+                                                buffer(bytearray([0x31,0x32,0x33,0x34,0x34]), 0, 4)), True);
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1),
+                                                buffer(bytearray([0x99,0x32,0x33,0x34,0x34]), 0, 4)), False);
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1), b'1234'), True);
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1),  '1234'), True);
+            self.assertEqual(areBytesEqual(buffer(bytearray([0x30,0x31,0x32,0x33,0x34]), 1), u'1234'), True);
 
 if __name__ == '__main__':
     unittest.main();
